@@ -331,10 +331,10 @@ unpackage_db (char *packname)
 	    DBOBJ key, content;
 
 	    key.data = (unsigned char *)STRING_VALUE (wispkey);
-	    key.length = 1 + strlen (key.data);
+	    key.length = 1 + strlen ((char *)key.data);
 
 	    content.data = (unsigned char *)contentdata;
-	    content.length = 1 + strlen (content.data);
+	    content.length = 1 + strlen ((char *)content.data);
 
 	    database_store (db, &key, &content);
 	    gc_wisp_objects ();
