@@ -37,7 +37,8 @@
 	   ;;; (modify-syntax-entry ?\) "-" mhtml-mode-syntax-table)
 	   (modify-syntax-entry ?\; "w" mhtml-mode-syntax-table)
 	   (modify-syntax-entry ?\& "w" mhtml-mode-syntax-table)
-	   (modify-syntax-entry ?\| "." mhtml-mode-syntax-table)))
+	   (modify-syntax-entry ?\| "." mhtml-mode-syntax-table)
+	   (modify-syntax-entry ?\; ". 12" mhtml-mode-syntax-table)))
 
 (defun mhtml-mode-variables (mhtml-syntax)
   (set-syntax-table mhtml-mode-syntax-table)
@@ -58,7 +59,7 @@
   (make-local-variable 'outline-regexp)
   (setq outline-regexp ";;; \\|<....")
   (make-local-variable 'comment-start)
-  (setq comment-start ";;; ")
+  (setq comment-start ";;;")
   (make-local-variable 'comment-start-skip)
   (setq comment-start-skip ";;;+ *")
   (make-local-variable 'comment-column)
@@ -76,6 +77,7 @@
     (define-key map [indent-region] '("Indent Region" . indent-region))
     (define-key map "\e\C-q" 'mhtml-indent-region)
     (define-key map "\eq" 'mhtml-indent-sexp)
+    (define-key map "\e\C-d" 'mhtml-show-documentation)
     (define-key map [indent-line] '("Indent Line" . indent-relative))
     (put 'eval-region 'menu-enable 'mark-active)
     (put 'comment-region 'menu-enable 'mark-active)
@@ -88,7 +90,8 @@
 (defconst mhtml-all-keywords
   '("set-var" "get-var" "unset-var" "subst-in-var" "subst-in-string"
     "package-vars" "package-names" "package-to-alist" "alist-to-package"
-    "increment" "if" "ifeq" "when" "not" "and" "or" "var-case" "while"
+    "increment" "if" "ifeq" "when" "whenelse" "elsewhen" "not" "and" "or"
+    "var-case" "while"
     "defmacro" "defsubst" "defun" "define-container" "define-function"
     "define-tag" "define-variable" "defvar" "debugging-on" "match" "include"
     "replace-page" "redirect" "cgi-encode" "cgi-decode" "cgi-exec" "comment"
@@ -116,6 +119,7 @@
     "sql::database-column-info" "sql::database-columns-info"
     "sql::database-query-info" "sql::database-set-options"
     "sql::cursor-get-column" "sql::query-get-column" "sql::sql-transact"
+    "tdbox"
     "example" "simple-fundoc" "complex-fundoc" "prog" "server-push"))
 
 (defun mhtml-build-font-lock-keywords (from-list)
@@ -216,7 +220,7 @@ rigidly along with this one."
     (concat result "\\)[ \t\n>]")))
 
 (defconst mhtml-complex-tags
-  '( "when" "while" "with" "defsubst" "defun" "defmacro" "defstruct"
+  '( "when" "whenelse" "while" "with" "defsubst" "defun" "defmacro" "defstruct"
      "define-tag" "define-function" "with-open-dmaild"
      "with-open-database" "with-open-stream" "server-push"
      "comment" "verbatim" "small-caps" "plain-text" "ptext" "section"
@@ -227,7 +231,7 @@ rigidly along with this one."
      "layout::page" "layout::left-border" "layout::right-border"
      "frameset" "msql::with-open-database" "odbc::with-open-database"
      "sql::with-open-database" "dbm::with-open-database" "def-pagelist"
-     "defweakmacro" "with-safe-environment")
+     "defweakmacro" "with-safe-environment" "tab-menu" "tdbox")
   "List of tag starts which are complex.")
 
 (defconst mhtml-complex-tag-start-regex
@@ -635,3 +639,42 @@ the output filename."
   (compile-internal
    (concat "mklib -o " (file-name-sans-extension file) ".lib " file)
    "No more errors."))
+
+;; Return a function which is called by the list containing point.
+;; If that gives no function, return a function whose name is around point.
+;; If that doesn't give a function, return nil.
+(defun mhtml-function-called-at-point ()
+  (save-excursion
+    (up-list -1)
+    (let ((beg (+ 1 (point))))
+      (goto-char beg)
+      (forward-sexp)
+      (buffer-substring beg (point)))))
+
+(defun mhtml-get-doc (string)
+  (save-excursion
+    (let ((buffer (get-buffer-create " *meta-doc*")))
+      (set-buffer buffer)
+      (delete-region (point-min) (point-max))
+      (call-process "/www/bin/show-doc.mhc" nil buffer nil string)
+      (buffer-substring (point-min) (point-max)))))
+
+(defun mhtml-show-documentation ()
+  "Display the full documentation of the current function."
+  (interactive)
+  (let ((function (mhtml-function-called-at-point))
+	doc)
+    (if function
+	(setq doc (mhtml-get-doc function)))
+    (if doc
+	(with-output-to-temp-buffer "*Help*"
+	  (save-excursion
+	    (set-buffer (get-buffer "*Help*"))
+	    (insert doc))
+	  (print-help-return-message)
+	  (save-excursion
+	    (set-buffer standard-output)
+	    (help-mode)
+	    ;; Return the text we displayed.
+	    (buffer-string)))
+      (message "You didn't specify a function"))))
