@@ -270,8 +270,24 @@ index.html\nhome.html");
     fatal ("Cannot read configuration file `%s'!", engine_config_path);
 #endif
 
+  /* Define a default function for handling a missing page.  This will be
+     used unless we find a configuration file. */
+  {
+    char *x = mhtml_evaluate_string
+      ("<defun mhttpd::default-document> <html> <body bgcolor=white> <dump-package mhttpd mhtml env> </body> </html> </defun>");
+    xfree (x);
+  }
+
   if (page && page->buffer)
-    page_process_page (page);
+    {
+      /* Kill the version of the debugging page function that we defined
+	 above. */
+      char *x = mhtml_evaluate_string ("<undef mhttpd::default-document>");
+      xfree (x);
+
+      /* Now run the engine config file. */
+      page_process_page (page);
+    }
   else
     {
       char *ignore = mhtml_evaluate_string ("<engine::initialize>");
@@ -833,12 +849,17 @@ process_request (void)
       fwrite (result->page->buffer + i, 1, result->page->bindex - i, stdout);
       fflush (stdout);
     }
+
+  page_process_page_internal (get_after_page_return_buffer ());
+
   if (result)
     {
       if (result->spec)
 	mhttpd_free_doc_spec (result->spec);
 
-      page_free_page (result->page);
+      if (result->page)
+	page_free_page (result->page);
+
       free (result);
     }
 }
