@@ -59,7 +59,36 @@ static PFunDesc func_table[] =
 };
 
 PACKAGE_INITIALIZER (initialize_array_functions)
-DOC_SECTION (ARRAYS)
+DEFINE_SECTION (ARRAYS, variables; arrays; multiple values, 
+"<meta-html> allows the use of <i>array</i> variables as well as single element variables.  In fact, all string variables in <meta-html> can be treated as array variables -- there is no special command for creating such variables.
+",
+"Array variable values are referenced by placing the array index directly after the variable name, enclosed in square brackets (<b>[</b> and <b>]</b>).  Thus, a reference to the 3rd element of the array <var foo> looks like:
+
+<example>
+   foo[3]
+</example>
+
+When an array reference is made without any containing index, the reference refers to the entire array.  So, to get the value of the entire array stored in <var foo>, you would write:
+
+<example>
+   <get-var foo[]>
+</example>
+
+In order to ease the writing of array references which rely on a variable index, a variable name seen as an array reference index is automatically looked up as if you had written <example code><get-var VAR></example>.  Finally, multiple values may be given in a <funref variables set-var> command by separating those values with newline characters.  The following sequence of commands illustrates the typical use of array variables.
+
+<complete-example>
+<set-var array[] =
+       \"value-zero
+        value-one
+        value-two
+        value-three\">
+    <set-var i=0>
+    <while <get-var array[i]>>
+      The value of array[<get-var i>] is `<get-var array[i]>'.<br>
+      <increment i>
+    </while>
+</complete-example>
+")
 
 DEFUN (pf_array_size, arrayvar,
 "Returns the number of elements in the array referenced by the
@@ -350,7 +379,7 @@ and, a subsequent execution of
   xfree (start_var);
 }
 
-DEFMACRO (pf_foreach, elementvar arrayvar &key start=x end=x step=x,
+DEFMACRO (pf_foreach, elementvar arrayvar &key start=x end=x step=x iter=var,
 "Perform <var body> with <var elementvar> bound to successive memebers
 of <var arrayvar>, starting with the element at <var start> (default
 0), and ending at <var end> (default <example code><array-size
@@ -384,6 +413,7 @@ or, we can produce a \"countdown\" with a negative value for <var step>:
   char *start_arg = mhtml_evaluate_string (get_value (vars, "start"));
   char *end_arg = mhtml_evaluate_string (get_value (vars, "end"));
   char *step_arg = mhtml_evaluate_string (get_value (vars, "step"));
+  char *iter_var = mhtml_evaluate_string (get_value (vars, "iter"));
   int start_index, end_index, step;
 
   if ((!empty_string_p (element_var)) && (!empty_string_p (array_var)))
@@ -431,8 +461,8 @@ or, we can produce a \"countdown\" with a negative value for <var step>:
 
 	  /* Final sanity check.  Make sure that START and END are within
 	     the bounds of the final array. */
-	  if ((step < 0) && start >= array->values_index)
-	    start = array->values_index;
+	  if ((step < 0) && start_index >= array->values_index)
+	    start_index = array->values_index - 1;
 
 	  if (((step < 0) && (end_index < start_index)) ||
 	      ((step > 0) && (end_index > start_index)))
@@ -450,10 +480,14 @@ or, we can produce a \"countdown\" with a negative value for <var step>:
 		  PAGE *code;
 		  int line = parser_current_lineno;
 
-		  symbol_free (element);
+		  if (element) symbol_free (element);
 		  element = symbol_intern (element_var);
 		  symbol_add_value (element, array_values[i]);
 		  code = page_copy_page (body);
+
+		  if (!empty_string_p (iter_var))
+		    mhtml_set_numeric_variable (iter_var, i);
+
 		  page_process_page_internal (code);
 		  parser_current_lineno = line;
 
@@ -484,6 +518,7 @@ or, we can produce a \"countdown\" with a negative value for <var step>:
   xfree (start_arg);
   xfree (end_arg);
   xfree (step_arg);
+  xfree (iter_var);
 }
 
 DEFUN (pf_array_concat, receiver &optional contributor...,
@@ -636,6 +671,7 @@ Finally, you may supply a sorting function, whose name is passed as <var sort-fu
 Examples:
 
 Given the array:
+<unset-var array>
 <set-var array[0] = 1
          array[1] = 2
          array[3] = 3
