@@ -3,4 +3,13 @@
 /*  Copyright (c) 1996 Brian J. Fox
     Author: Brian J. Fox (bfox@ai.mit.edu) Wed Jan 29 11:51:54 1997.  */
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 extern void bootstrap_metahtml (int call_initializer_p);
+
+#if defined (__cplusplus)
+}
+#endif
