@@ -58,6 +58,10 @@ static PFunDesc func_table[] =
 PACKAGE_INITIALIZER (initialize_cgi_exec_functions)
 DOC_SECTION (FILE-OPERATORS)
 
+DEFVAR (mhtml::exec-path,
+"A colon-seperated list of directories in which <Meta-HTML> searches
+for executables when executing the <funref FILE-OPERATORS cgi-exec> tag.")
+
 #if defined (NEW_CGI_EXEC)
 static char **
 make_environ (void)
