@@ -33,6 +33,11 @@
 #  include <config.h>
 #endif
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -516,7 +521,8 @@ forms_get_tag_value_in_package (Package *package, char *tag)
 	    }
 	  else
 	    {
-	      if (value_index < symbol->values_index)
+	      if ((value_index < symbol->values_index) &&
+		  (value_index > -1))
 		value = symbol->values[value_index];
 	    }
 	}
@@ -704,8 +710,11 @@ forms_set_tag_value_in_package (Package *package, char *tag, char *value)
 	}
       else
 	{
-	  free (symbol->values[value_index]);
-	  symbol->values[value_index] = strdup (value);
+	  if (value_index > -1)
+	    {
+	      free (symbol->values[value_index]);
+	      symbol->values[value_index] = strdup (value);
+	    }
 	}
     }
 
@@ -743,4 +752,6 @@ forms_gc_pointers (void)
   forms_gc_list_index = 0;
 }
 
-  
+#if defined (__cplusplus)
+}
+#endif
