@@ -69,23 +69,30 @@ static void pf_sql_transact (PFunArgs);
 #define GSQL_NO_DATA_FOUND		100
 
 /* Standard GSQL datatypes (agree with ANSI type numbering). */
-#define GSQL_CHAR			1
-#define GSQL_NUMERIC 			2
-#define GSQL_DECIMAL 			3
-#define GSQL_INTEGER 			4
-#define GSQL_SMALLINT			5
-#define GSQL_FLOAT			6
-#define GSQL_REAL			7
-#define GSQL_DOUBLE			8
-#define GSQL_VARCHAR 			12
+#define GSQL_CHAR	1
+#define GSQL_NUMERIC 	2
+#define GSQL_DECIMAL 	3
+#define GSQL_INTEGER 	4
+#define GSQL_SMALLINT	5
+#define GSQL_FLOAT	6
+#define GSQL_REAL	7
+#define GSQL_DOUBLE	8
+#define GSQL_VARCHAR 	12
+#define GSQL_BOOLEAN	16
 
 /* These are not standard ANSI */
-#define GSQL_IDENT          100
-#define GSQL_NULL           101
-#define GSQL_IDX            102
-#define GSQL_SYSVAR         103
-#define GSQL_UINT           104
-#define GSQL_ANY            105
+#define GSQL_IDENT	100
+#define GSQL_NULL	101
+#define GSQL_IDX	102
+#define GSQL_SYSVAR	103
+#define GSQL_UINT	104
+#define GSQL_ANY	105
+
+/* PostGres SQL extensions. */
+#define GSQL_DATE	200
+#define GSQL_TIME	201
+#define GSQL_MONEY	202
+
 
 /* MYSQL extensions. (tamsky@as.ucsb.edu: Sat Oct 11 11:55:38 1997) */
 #define GSQL_TINY_BLOB      249
@@ -248,6 +255,10 @@ dsn_lookup (char *key, char *str)
 #  include "mysqlfuncs.c"
 #endif
 
+#if defined (COMPILING_PGSQLFUNCS)
+#  include "pgsqlfuncs.c"
+#endif
+
 #if defined (COMPILING_ODBCFUNCS)
 #  include "odbcfuncs.c"
 #endif
@@ -447,7 +458,7 @@ gsql_database_set_sql_escape_character (Database *db, char *escape_code)
 static char
 gsql_database_sql_escape_character (Database *db)
 {
-  return db->sql_escape_char;
+  return (db->sql_escape_char);
 }
 
 /* If this variable is set, automatically truncate strings to max
@@ -469,7 +480,7 @@ gsql_database_set_truncate_columns (Database *db, char *value)
 static int
 gsql_database_sql_truncate_columns (Database *db)
 {
-  return db->sql_truncate_columns;
+  return (db->sql_truncate_columns);
 }
 
 /* If this variable is set, then database-query and next-record will
@@ -1942,7 +1953,8 @@ bprintf_gsql_escape (BPRINTF_BUFFER *out, char *src, gsql_field *field,
   while ((c = *p++) != 0)
     {
       i++;
-      if ((maxlength != 0) && (i > maxlength)) return;
+      if ((truncate_p != 0) && ((maxlength != 0) && (i > maxlength)))
+	return;
 
       /* Oracle doesn't need parens quoted, but mSQL and MySQL do. */
 #if defined (COMPILING_MSQLFUNCS) || defined (COMPILING_MYSQLFUNCS)
@@ -2488,7 +2500,8 @@ pf_database_save_package (PFunArgs)
 {
   Database *db = get_dbref (vars);
   char *key     = mhtml_evaluate_string (get_positional_arg (vars, 1));
-  char *keyname = mhtml_evaluate_string (get_one_of (vars, "keyname", "key", (char *)NULL));
+  char *keyname = mhtml_evaluate_string (get_one_of (vars, "keyname", "key",
+						     (char *)NULL));
   char *table   = mhtml_evaluate_string (get_value (vars, "table"));
   char *method   = mhtml_evaluate_string (get_value (vars, "method"));
   BPRINTF_BUFFER *query = (BPRINTF_BUFFER *) NULL;
@@ -2554,6 +2567,12 @@ pf_database_save_package (PFunArgs)
 	if (!empty_string_p (packname))
 	  {
 	    symbols = symbol_package_symbols (packname);
+
+	    if (symbols == (Symbol **)NULL)
+	      {
+		gsql_save_error_message (db, "package contains no variables");
+		errors_found++;
+	      }
 	  }
 	else
 	  {
