@@ -65,8 +65,24 @@ static PFunDesc func_table[] =
 };
 
 PACKAGE_INITIALIZER (initialize_msql_functions)
-DOC_SECTION (MSQL-DATABASE-EXTENSIONS)
-DEFMACROX (pf-msql::with_open_database, ,
+DEFINE_SECTION (MSQL-DATABASE-EXTENSIONS, database; SQL; Minerva,
+"<Meta-HTML> can use the <b>mSQL</b> database engine (residing in the
+dynamic module <code>modmsql.so</code>) when requested to by the
+<funref generic-sql-interface sql::set-database-type> command.
+The mSQL extensions allow true SQL database interaction at many levels,
+providing a clean and flexible abstraction to an mSQL database.", "")
+
+DEFVAR (msql::msql-error-message,
+"An array of strings containing any error messages generated by the
+last call to the mSQL database.  Don't use this variable, use
+the function <funref generic-sql-interface sql::sql-error-message> instead.")
+
+DEFVAR (msql::recent-query,
+"The last query sent to the mSQL database for execution.  Don't use this
+variable, use the function <funref generic-sql-interface sql::recent-query>
+instead.")
+
+DEFMACROX (pf_msql::with_open_database, ,
 "See <funref generic-sql-interface sql::with-open-database>.")
 DEFUNX (pf_msql::database_exec_query, ,
 "See <funref generic-sql-interface sql::database-exec-query>.")
