@@ -65,7 +65,23 @@ static PFunDesc func_table[] =
 };
 
 PACKAGE_INITIALIZER (initialize_mysql_functions)
-DOC_SECTION (MYSQL-DATABASE-EXTENSIONS)
+DEFINE_SECTION (MYSQL-DATABASE-EXTENSIONS, database; SQL; Minerva,
+"<Meta-HTML> can use the <b>MySQL</b> database engine (residing in the
+dynamic module <code>modmysql.so</code>) when requested to by the
+<funref generic-sql-interface sql::set-database-type> command.
+The MySQL extensions allow true SQL database interaction at many levels,
+providing a clean and flexible abstraction to a MySQL database.", "")
+
+DEFVAR (mysql::mysql-error-message,
+"An array of strings containing any error messages generated by the
+last call to the MySQL database.  Don't use this variable, use
+the function <funref generic-sql-interface sql::sql-error-message> instead.")
+
+DEFVAR (mysql::recent-query,
+"The last query sent to the MySQL database for execution.  Don't use this
+variable, use the function <funref generic-sql-interface sql::recent-query>
+instead.")
+
 DEFMACROX (pf_mysql::with_open_database, ,
 "See <funref generic-sql-interface sql::with-open-database>.")
 DEFUNX (pf_mysql::database_exec_query, ,
