@@ -77,7 +77,39 @@ static PFunDesc func_table[] =
 };
 
 PACKAGE_INITIALIZER (initialize_package_functions)
-DOC_SECTION (PACKAGES)
+DEFINE_SECTION (PACKAGES, variables; module; package; group, 
+"<i>Packages</i> are repositories which contain symbols and their values.
+
+Each time a symbol is referenced, a package must first be found, and
+then the symbol may be found within that package. You indicate which
+package to look the symbol up in by supplying a package <i>prefix</i>.
+When the prefix is not supplied, the symbol is looked up in the
+current package.",
+"For example, a full reference to the symbol <var bar> stored in the
+<var foo> package looks like: 
+
+<example>
+   FOO::BAR
+</example>
+
+There are very few commands specifically for dealing with packages,
+because most of the operations are performed implicitly, rather than
+explicitly.  To create a package, simply give the package name as part
+of the symbol, in the place where the symbol is normally used.
+
+<example>
+   <set-var foo::bar = \"Hello\">
+</example>
+
+This has the effect of creating the package <var foo> if it didn't
+already exist.
+
+The majority of the functions documented here perform package maintenance, as
+opposed to variable manipulation.  There are functions for querying a package
+about its contens, for deleting an entire package, for exporting and importing
+packages to and from sessions, for copying the contents of packages, and for
+converting packages from an internal representation to a printable
+representation, called an <i>alist</i>.")
 
 DEFUN (pf_package_names, ,
 "Returns a newline separated list of the all of the named packages
@@ -496,9 +528,8 @@ the variable named by <var varname>, in the association list specified by
 fact exist.
 
 The existence of a variable has nothing to do with its value -- a
-variable exists if it has been created with <funref variables
-set-var>, and doesn't exist after it has been unset with <funref
-variables unset-var>.")
+variable exists if it is present within the list, whether or not it
+has a value.")
 {
   Package *alist = get_alist_var (vars);
   char *arg = mhtml_evaluate_string (get_positional_arg (vars, 1));
