@@ -132,9 +132,55 @@ static PFunDesc func_table[] =
 
 PACKAGE_INITIALIZER (initialize_pagefunc_functions)
 
+DEFINE_SECTION (PAGE-VARIABLES, , "Several variables are predefined by
+<meta-html>, and made available to the page writer.  In addition to
+the contents of the Unix environment (which includes the server-side
+variables <varref ENV::SERVER_NAME>, <varref ENV::PATH_INFO>, etc.)
+variables describing the location of the current document in various
+ways are also defined.  ", "The following short list is a quick look
+at typical values of the most commonly referenced page variables.
+
+When the URL is ``<i><get-var mhtml::full-url></i>'', the following
+variables are set to:
+
+<defun showvar var>
+  <tr>
+    <td align=right> <varref <upcase <get-var var>>>: </td>
+    <td align=left>
+      <i><get-var <get-var var>></i><br>
+    </td>
+  </tr>
+</defun>
+
+<table>
+  <showvar mhtml::path-info>
+  <showvar mhtml::http-to-host>
+  <showvar mhtml::current-doc>
+  <showvar mhtml::include-prefix>
+  <showvar mhtml::relative-prefix>
+  <showvar mhtml::location>
+  <showvar mhtml::location-sans-sid>
+  <showvar mhtml::full-url>
+  <showvar mhtml::full-url-sans-sid>
+  <showvar mhtml::url-to-dir>
+  <showvar mhtml::url-to-dir-sans-sid>
+  <showvar mhtml::http-prefix>
+  <showvar mhtml::http-prefix-sans-sid>
+</table>")
+
 DOC_SECTION (ARITHMETIC-OPERATORS)
 static int randomize_called = 0;
 
+static unsigned int
+random_seed (void)
+{
+  unsigned int seed = (unsigned int) getpid ();
+  time_t now = time ((time_t *)NULL);
+
+  seed = seed | (unsigned int)now;
+  return (seed);
+}
+
 DEFUN (pf_randomize, &optional seed,
 "Sets the pseudo-random number generator seed to <var seed>.  The next
 call to <funref arithmetic-operators random> uses this seed value to
@@ -149,7 +195,7 @@ Examples:
   <random 100> --> 28
 </example>")
 {
-  unsigned int seed = (unsigned int)getpid ();
+  unsigned int seed = random_seed ();
   char *user_seed = mhtml_evaluate_string (get_positional_arg (vars, 0));
 
   if (user_seed)
@@ -185,7 +231,7 @@ Also see <funref arithmetic-operators randomize>.")
 
   if (!randomize_called)
     {
-      srandom ((unsigned int)getpid ());
+      srandom (random_seed ());
       randomize_called++;
     }
 
@@ -199,7 +245,29 @@ Also see <funref arithmetic-operators randomize>.")
   bprintf_insert (page, start, "%d", result);
 }
 
-DOC_SECTION (LANGUAGE-OPERATORS)
+DEFINE_SECTION (LANGUAGE-OPERATORS, self-modif; language;
+self-referential, "<Meta-HTML> provides various functions and
+variables which are used to operate on or within the language
+environment itself, rather than within the application environment.
+
+Such constructs make it possible to create self-modifying code, to
+change the way in which a builtin function operates, and other such
+language dependent features.", "")
+
+DEFVAR (MHTML::INHIBIT-COMMENT-PARSING, "When this variable is set to
+a non-empty value, the triple semi-colon comment feature is disabled.
+
+You may need this for complex applications where you are deliberately
+shipping the source of a <meta-html> document to the client.")
+
+DEFVAR (MHTML::VERSION, "Contains the version identification of the
+currently running interpreter.  This variable is available whereever
+the interpreter is running, including the <meta-html> server, Engine,
+and in <code>mdb</code>.
+
+<complete-example>
+<get-var mhtml::version>
+</complete-example>")
 
 DEFUN (pf_make_identifier, limit,
 "Create an identifier that is <var limit> characters in length.
@@ -270,7 +338,19 @@ Example:
   /* Contents already deleted by caller. */
 }
 
-DOC_SECTION (PAGE-OPERATORS)
+DEFINE_SECTION (PAGE-OPERATORS, page commands; page search; search;
+insert; modify page,
+"<meta-html> contains a few commands which
+operate on the entire <i>page</i> as it currently exists.
+
+You may find, modify, or delete text which has already been looked at by the interpreter, as well as text which hasn't yet been looked at.
+
+Most of the time, you won't need to use such commands.  They can make
+it hard for other people to understand the sequence of code in your
+page, and they can sometimes produce unexpected results which are
+difficult to debug because much of the information has been modified.
+", "")
+
 static int include_recursive_calls = 0;
 
 DEFUN (pf_include, webpath &key alt=altcode verbatim=true,
@@ -300,14 +380,20 @@ value is given, then that value is placed into the page instead.")
       char *incpref = pagefunc_get_variable ("%%::incpref");
       char *relpref = pagefunc_get_variable ("%%::relpref");
 
+      if (debug_level > 10)
+	page_debug ("<include %s> [relpref: %s]",
+		    arg ? arg : "\"\"",
+		    relpref ? relpref : "\"\"");
+		    
       if (!incpref) incpref = pagefunc_get_variable ("mhtml::include-prefix");
       if (!relpref) relpref = pagefunc_get_variable ("mhtml::relative-prefix");
 
       if (arg != (char *)NULL)
 	{
 	  PAGE *file_contents = (PAGE *)NULL;
+	  char *wr = (char *)NULL;
 
-	  pathname = mhtml_canonicalize_file_name (arg, incpref, relpref);
+	  pathname = mhtml_canonicalize_file_name (arg, incpref, relpref, &wr);
 
 	  if (!empty_string_p (pathname))
 	    file_contents = page_read_template (pathname);
@@ -343,7 +429,7 @@ value is given, then that value is placed into the page instead.")
 	      if (found && !verbatim_p)
 		{
 		  bprintf_insert (file_contents, 0, "<*parser*::push-file %s>",
-				  page_last_read_filename);
+				  wr);
 		  bprintf (file_contents, "<*parser*::pop-file>");
 		}
 
@@ -352,42 +438,13 @@ value is given, then that value is placed into the page instead.")
 
 	      if (verbatim_p)
 		*newstart += file_contents->bindex;
-	      else
-		{
-#if defined (MHTML_INCLUDE_IS_RELATIVE)
-		  if (!empty_string_p (relpref))
-		    {
-		      char *temp = strstr (pathname, incpref);
-
-		      if (temp != (char *)NULL)
-			{
-			  char *last;
-
-			  temp += strlen (incpref);
-			  last = strrchr (temp, '/');
-
-			  if (last != (char *)NULL)
-			    {
-			      *last = '\0';
-
-			      /* Insert instructions to restore the state. */
-			      bprintf_insert
-				(page, start + file_contents->bindex,
-				 "<set-var %%%%::relpref=%s>", relpref);
-			      pagefunc_set_variable ("%%::relpref", temp);
-
-			    }
-			}
-		    }
-#endif /* MHTML_INCLUDE_IS_RELATIVE */
-		}
 
 	      page_free_page (file_contents);
 	    }
 	  free (arg);
 	}
     }
-  if (pathname != (char *)NULL) free (pathname);
+  xfree (pathname);
   include_recursive_calls--;
 }
 
@@ -499,7 +556,10 @@ wrap_by_character_class (char *string, int small_caps_p, int leave_braces,
   return (result);
 }
 
-DOC_SECTION (HTML-HELPERS)
+DEFINE_SECTION (HTML-HELPERS, HTML; helper; convenience, 
+"The following functions all produce HTML as output and are defined in
+order to help with the creation of forms and tables.", "")
+
 DEFMACRO (pf_small_caps, &key upper=size lower=size other=size,
 "Modify the characters in <var body> raising lower-case
 characters to upper-case, and changing the size of fonts as directed.
@@ -672,7 +732,8 @@ cause an immediate redirection to the specified page.
 
 <var url> may be expressed in relative terms, in absolute terms, or as
 a complete URI specification.  For example, assume the current
-document is located at `http://www.metahtml.com/subdir/doc.mhtml', and
+document is located at
+<example code>http://www.metahtml.com/subdir/doc.mhtml</example>, and
 the following redirections are seen:
 <example>
   <redirect ../foo.mhtml>     --> http://www.metahtml.com/foo.mhtml
@@ -898,12 +959,20 @@ Contrast this with <funref page-operators comment>.")
     }
 }
 
-DOC_SECTION (LANGUAGE-OPERATORS)
+DEFINE_SECTION (DATES-AND-TIMES,
+		canonical date; time functions; clock; zone,
+"<Meta-HTML> provides a number of pre-defined functions for manipulating
+date and time objects.  Along with the primitive builtins of <tag date>
+and <tag time>, there are several functions defined with the prefix of
+<code>date</code> which can be used to parse dates, convert between time
+and date formats, format time strings in a variety of ways, and even
+produce HTML-based calendars and the like.", "")
+
 DEFUN (pf_time, ,
 "Returns the current local time, as measured in seconds since the
 epoch (12 midnight, December 31, 1969).
 
-This is often useful as input to the <funref language-operators date>
+This is often useful as input to the <funref dates-and-times date>
 operator.  Here is how one might find out the printable date for the
 time 10 hours from now.
 
@@ -938,7 +1007,7 @@ Examples:
 <date <add <time> 60>>  --> Thu Nov 13 10:53:07 1997
 </example>
 
-Also see the <funref language-operators time> function.")
+Also see the <funref dates-and-times time> function.")
 {
   char *tstring = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *gmt = get_value (vars, "gmt");
@@ -956,6 +1025,7 @@ Also see the <funref language-operators time> function.")
   xfree (tstring);
 }
 
+DOC_SECTION (LANGUAGE-OPERATORS)
 DEFUN (pf_pid, ,
 "Returns the process ID of the currently executing Meta-HTML
 interpreter, on those systems which have process IDs.")
