@@ -686,6 +686,22 @@ empty_string_p (char *string)
   return (result);
 }
 
+void
+mhtml_set_numeric_variable (char *name, int value)
+{
+  static char rep[128];
+  sprintf (rep, "%d", value);
+  pagefunc_set_variable (name, rep);
+}
+
+void
+mhtml_set_numeric_variable_in_package (Package *p, char *name, int value)
+{
+  static char rep[128];
+  sprintf (rep, "%d", value);
+  forms_set_tag_value_in_package (p, name, rep);
+}
+
 /* Read STRING, and convert the contents to a list of variables in PACKAGE. */
 Package *
 alist_to_package (char *string)
@@ -1531,7 +1547,12 @@ mhtml_add_user_function (int type, char *name, char *body, Package *vars)
 
   if (type == user_DEFUN)
     {
-      body_whitespace = "delete";
+      if ((!empty_string_p (body_whitespace)) &&
+	  (strcasecmp (body_whitespace, "keep") == 0))
+	body_whitespace = (char *)NULL;
+      else
+	body_whitespace = "delete";
+
       if (!wrapper_packname)
 	wrapper_packname = strdup ("local");
     }
@@ -1804,6 +1825,9 @@ mhtml_let (PFunArgs)
 	symbol_copy (from, pack);
     }
 
+  if (debug_level)
+    mhtml_set_numeric_variable_in_package (pack, ":debug-level", debug_level);
+
   /* Execute the body. */
   {
     PageEnv *page_environ;
@@ -1856,6 +1880,13 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 {
   PAGE *subber = (PAGE *)NULL;
 
+  if (uf->debug_level == 21)
+    {
+      /* Magic debugging location.  Set a GDB breakpoint here and use
+	 <debugging-on function=21> */
+      uf->debug_level = 21;
+    }
+
   if (!empty_string_p (uf->body))
     {
       register int i = 0, j;
@@ -2073,6 +2104,10 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 				{
 				  char *setval = gather->buffer;
 
+				  /* Get rid of extra space at buffer end. */
+				  gather->bindex--;
+				  gather->buffer[gather->bindex] = '\0';
+
 				  if (!unevalled)
 				    {
 				      setval = mhtml_evaluate_string (setval);
@@ -2153,6 +2188,10 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 
 		  symbol_push_package (local);
 
+		  /* Allow user level debugging to take place. */
+		  if (debug_level)
+		    mhtml_set_numeric_variable (":debug-level", debug_level);
+
 		  x = mhtml_evaluate_string (subber->buffer);
 
 		  if (!empty_string_p (x))
@@ -2211,7 +2250,8 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 /* Canonicalize the filename INPUT such that it is a complete and
    valid path to a file. */
 char *
-mhtml_canonicalize_file_name (char *input, char *docroot, char *relpref)
+mhtml_canonicalize_file_name (char *input, char *docroot, char *relpref,
+			      char **web_relative_name)
 {
   register int i;
   char *result = (char *)NULL;
@@ -2221,6 +2261,8 @@ mhtml_canonicalize_file_name (char *input, char *docroot, char *relpref)
   int relpref_len = relpref ? strlen (relpref) : 0;
   int input_len = input ? strlen (input) : 0;
   int maxlen = 10 + input_len + docroot_len + relpref_len;
+  static char *saved_webname = (char *)NULL;
+  static int saved_webname_len = 0;
 
   if (input == (char *)NULL)
     return (input);
@@ -2286,6 +2328,14 @@ mhtml_canonicalize_file_name (char *input, char *docroot, char *relpref)
       }
   }
 
+  /* Save the web relative pathname here. */
+  if (saved_webname_len + 2 < strlen (workbuff))
+    saved_webname = (char *)xrealloc
+      (saved_webname, (saved_webname_len = 10 + strlen (workbuff)));
+
+  strcpy (saved_webname, workbuff);
+  *web_relative_name = saved_webname;
+
 #if defined (HAVE_GETPWNAM)
   /* If username expansion is being allowed, we allow it to work here
      as well. */
@@ -2373,6 +2423,7 @@ mhtml_canonicalize_file_name_argument (char *argument)
 {
   char *temp = mhtml_evaluate_string (argument);
   char *result = (char *)NULL;
+  char *ign;
 
   if (!empty_string_p (temp))
     {
@@ -2386,7 +2437,7 @@ mhtml_canonicalize_file_name_argument (char *argument)
 	  if (!relpref)
 	    relpref = pagefunc_get_variable ("mhtml::relative-prefix");
 
-	  result = mhtml_canonicalize_file_name (temp, incpref, relpref);
+	  result = mhtml_canonicalize_file_name (temp, incpref, relpref, &ign);
 	}
       else
 	result = temp;
@@ -2607,51 +2658,51 @@ mhtml_base64encode (char *data, int length, int shortlines)
   return (result);
 }
 
-
 int
-number_p (char *string)
+float_p (char *string)
 {
-  register int i;
   int result = 0;
 
-  if (string == (char *)NULL)
-    return (0);
-
-  /* Skip leading whitespace. */
-  for (i = 0; whitespace (string[i]); i++);
-
-  if (string[i])
+  if (!empty_string_p (string))
     {
-      int decimal_seen = 0;
+      char *endptr;
+      double value = strtod (string, &endptr);
 
-      result = 1;
-
-      if ((string[i] == '-') || (string[i] == '+'))
-	i++;
-
-      for (; string[i]; i++)
-	{
-	  if (string[i] == '.')
-	    {
-	      if (decimal_seen)
-		{
-		  result = 0;
-		  break;
-		}
-	      else
-		decimal_seen++;
-	    }
-	  else if (!isdigit (string[i]))
-	    {
-	      result = 0;
-	      break;
-	    }
-	}
+      if (*endptr == '\0')
+	result = 1;
+      else
+	/* Make sure that GCC doesn't optimize away the strtod call. */
+	value += 1;
     }
 
   return (result);
 }
 
+int
+integer_p (char *string, int base)
+{
+  int result = 0;
+
+  if (!empty_string_p (string))
+    {
+      char *endptr;
+      long value = strtol (string, &endptr, base);
+
+      if ((endptr[0] == '\0') || ((endptr[0] == '.') && (endptr[1] == '\0')))
+	result = 1;
+      else
+	/* Make sure that GCC doesn't optimize away the strtod call. */
+	value += 1;
+    }
+  return (result);
+}
+
+int
+number_p (char *string)
+{
+  return ((integer_p (string, 0)) || (float_p (string)));
+}
+  
 #if defined (__cplusplus)
 }
 #endif
