@@ -28,6 +28,7 @@
    93101  */
 
 #include <pgsql/libpq-fe.h>
+#include <pgsql/config.h>
 
 /****************************************************************/
 
@@ -35,7 +36,7 @@
 #define DEFAULT_SQL_TRUNCATE_COLUMNS  1
 #define DEFAULT_SQL_PREFIX_TABLENAMES 0
 
-static PFunDesc func_table[]
+static PFunDesc func_table[] =
 {
   { "PGSQL::WITH-OPEN-DATABASE",	1, 0, pf_with_open_database },
   { "PGSQL::DATABASE-EXEC-QUERY",	0, 0, pf_database_exec_query },
@@ -64,6 +65,74 @@ static PFunDesc func_table[]
 };
 
 PACKAGE_INITIALIZER (initialize_pgsql_functions)
+DEFINE_SECTION (PGSQL-DATABASE-EXTENSIONS, database; SQL; PostGres,
+"<Meta-HTML> can use the <b>PostGres</b> database engine (residing in the
+dynamic module <code>modpgsql.so</code>) when requested to by the
+<funref generic-sql-interface sql::set-database-type> command.
+The PostGres extensions allow true SQL database interaction at many levels,
+providing a clean and flexible abstraction to a PostGres database.",
+"At the time of this writing, the PostGres implementation is in <i>beta</i>
+release -- the speed of the implementation is not as good as the mSQL, MySQL,
+or ODBC implementation.  We expect this to change in the near future")
+
+DEFVAR (pgsql::pgsql-error-message,
+"An array of strings containing any error messages generated by the
+last call to the PostGres database.  Don't use this variable, use
+the function <funref generic-sql-interface sql::sql-error-message> instead.")
+
+DEFVAR (pgsql::recent-query,
+"The last query sent to the PostGres database for execution.  Don't use this
+variable, use the function <funref generic-sql-interface sql::recent-query>
+instead.")
+
+DEFMACROX (pf_pgsql::with_open_database, ,
+"See <funref generic-sql-interface sql::with-open-database>.")
+DEFUNX (pf_pgsql::database_exec_query, ,
+"See <funref generic-sql-interface sql::database-exec-query>.")
+DEFUNX (pf_pgsql::database_exec_sql, ,
+"See <funref generic-sql-interface sql::database-exec-sql>.")
+DEFUNX (pf_pgsql::database_next_record, ,
+"See <funref generic-sql-interface sql::database-next-record>.")
+DEFUNX (pf_pgsql::database_save_record, ,
+"See <funref generic-sql-interface sql::database-save-record>.")
+DEFUNX (pf_pgsql::database_delete_record, ,
+"See <funref generic-sql-interface sql::database-delete-record>.")
+DEFUNX (pf_pgsql::database_load_record, ,
+"See <funref generic-sql-interface sql::database-load-record>.")
+DEFUNX (pf_pgsql::database_save_package, ,
+"See <funref generic-sql-interface sql::database-save-package>.")
+DEFUNX (pf_pgsql::database_num_rows, ,
+"See <funref generic-sql-interface sql::database-num-rows>.")
+DEFUNX (pf_pgsql::database_affected_rows, ,
+"See <funref generic-sql-interface sql::database-affected-rows>.")
+DEFUNX (pf_pgsql::database_set_pos, ,
+"See <funref generic-sql-interface sql::database-set-pos>.")
+DEFUNX (pf_pgsql::database_query, ,
+"See <funref generic-sql-interface sql::database-query>.")
+DEFUNX (pf_pgsql::host_databases, ,
+"See <funref generic-sql-interface sql::host-databases>.")
+DEFUNX (pf_pgsql::database_tables, ,
+"See <funref generic-sql-interface sql::database-tables>.")
+DEFUNX (pf_pgsql::database_tables_info, ,
+"See <funref generic-sql-interface sql::database-tables_info>.")
+DEFUNX (pf_pgsql::database_tables_info, ,
+"See <funref generic-sql-interface sql::database-tables-info>.")
+DEFUNX (pf_pgsql::database_columns, ,
+"See <funref generic-sql-interface sql::database-columns>.")
+DEFUNX (pf_pgsql::database_column_info, ,
+"See <funref generic-sql-interface sql::database-column-info>.")
+DEFUNX (pf_pgsql::database_columns_info, ,
+"See <funref generic-sql-interface sql::database-columns-info>.")
+DEFUNX (pf_pgsql::database_query_info, ,
+"See <funref generic-sql-interface sql::database-query-info>.")
+DEFUNX (pf_pgsql::database_set_options, ,
+"See <funref generic-sql-interface sql::database-set-options>.")
+DEFUNX (pf_pgsql::cursor_get_column, ,
+"See <funref generic-sql-interface sql::cursor-get-column>.")
+DEFUNX (pf_pgsql::query_get_column, ,
+"See <funref generic-sql-interface sql::query-get-column>.")
+DEFUNX (pf_pgsq::sql_transact, ,
+"See <funref generic-sql-interface sql::sql-transact>.")
 
 /****************************************************************
  * The Database object:
@@ -90,16 +159,18 @@ typedef struct
 
 typedef struct
 {
-  char *value;   /* value of this field. */
-  int   fsize;   /* length of the field, -1 if var-length */
-
-  /* These hold onto specific information about the field. */
+  char *name;		/* Field name. */
+  char *table;		/* The name of the table that this field belongs to. */
+  char *value;		/* The value of this field. */
+  int  fsize;		/* Size of the field, -1 if of variable length */
+  int  ftype;		/* PostGres internal type. */
   char *qualifier;
   char *owner;
-  char *typename;
+  char *type_name;
   int precision;
   int scale;
   int radix;
+  int nullable;
 } gsql_field;
 
 typedef struct
@@ -178,6 +249,21 @@ gsql_save_error_message (Database *db, char *msg)
     }
 }
 
+static char *
+my_pgtty (void)
+{
+  static char *pgtty = (char *)NULL;
+  static int looked_up = 0;
+
+  if (!looked_up)
+    pgtty = (char *)getenv ("PGTTY");
+
+  if (pgtty == (char *)NULL)
+    return ("/dev/null");
+  else
+    return (pgtty);
+}
+
 static int
 gsql_number_of_rows (gsql_result *result)
 {
@@ -219,7 +305,7 @@ gsql_fetch_row (gsql_result *result)
       numfields = PQnfields (result->postgres_result);
       for (i = 0; i < numfields; i++)
 	{
-	  gsql_field * currfield = result->fields[i];
+	  gsql_field *currfield = result->fields[i];
 	  free (currfield->value);
 	  currfield->value = strdup (PQgetvalue (result->postgres_result,
 						 result->current_row, i));
@@ -229,7 +315,7 @@ gsql_fetch_row (gsql_result *result)
       return (GSQL_SUCCESS);
     }
   else
-    return GSQL_NO_DATA_FOUND;
+    return (GSQL_NO_DATA_FOUND);
 }
 
 static gsql_field *
@@ -244,28 +330,15 @@ gsql_fetch_field (gsql_result *result, int i)
 static int
 gsql_query (Database *db, char *query, int save_errors_p)
 {
-  ExecStatusType query_result_status;
+  ExecStatusType query_result_status = PGRES_FATAL_ERROR;
 
-  /* Free up memory from our last query (assuming we -had-
-     a last query, that is ... */
-
-  /*  FIXME: is this a memory leak?  I think this result
-	is somehow getting freed up in gsql_free_result()
-	but I don't feel like chasing the pointers.  Uncomment
-	the following lines and watch it segfault ... :) */
-
-  /*  if(db->postgres_last_query != (PGresult *)NULL)
-    {
-      PQclear (db->postgres_last_query);
-      db->postgres_last_query = (PGresult *)NULL;
-    }
-  */
-
-  /* Store the results from the new query */
+  /* Store the results from the query. */
   db->postgres_last_query = PQexec (db->sock, query);
-  query_result_status = PQresultStatus (db->postgres_last_query);
 
-  pagefunc_set_variable ("postgres::recent-query", query);
+  if (db->postgres_last_query != (PGresult *)NULL)
+    query_result_status = PQresultStatus (db->postgres_last_query);
+
+  pagefunc_set_variable ("pgsql::recent-query", query);
 
   if ((query_result_status == PGRES_BAD_RESPONSE) ||
       (query_result_status == PGRES_NONFATAL_ERROR) ||
@@ -280,21 +353,32 @@ gsql_query (Database *db, char *query, int save_errors_p)
     return (GSQL_SUCCESS);
 }
 
+static void
+gsql_free_field (gsql_field *f)
+{
+  if (f != (gsql_field *)NULL)
+    {
+      xfree (f->value);
+      xfree (f->name);
+      xfree (f->table);
+      xfree (f->qualifier);
+      xfree (f->owner);
+      xfree (f->type_name);
+      free (f);
+    }
+}
+
 static void
 gsql_free_result (gsql_result *result)
 {
-  int i;
+  register int i;
   PGresult *pr = result->postgres_result;
 
   if (result->fields != (gsql_field **) NULL)
     {
-      i = 0;
-      while (result->fields[i] != (gsql_field *) NULL)
-	{
-	  /* Free the gsql_field structs.  */
-	  free (result->fields[i]);
-	  i++;
-	}
+      for (i = 0; result->fields[i] != (gsql_field *) NULL; i++)
+	gsql_free_field (result->fields[i]);
+
       free (result->fields);
     }
 
@@ -339,13 +423,17 @@ static void
 initialize_gsql_field (gsql_field *gfield)
 {
   /* PostGreSQL-specific initialization ... */
-  gfield->value   = (char *) NULL;
-  gfield->fsize   = 0;
+  gfield->value = (char *) NULL;
+  gfield->fsize = 0;
+  gfield->ftype = 0;
+  gfield->nullable = 0;;
 
   /* gsql standard stuff */
+  gfield->name = (char *)NULL;
+  gfield->table = (char *)NULL;
   gfield->qualifier = (char *) NULL;
   gfield->owner     = (char *) NULL;
-  gfield->typename  = (char *) NULL;
+  gfield->type_name  = (char *) NULL;
   gfield->precision = 0;
   gfield->scale     = 0;
   gfield->radix     = 0;
@@ -364,22 +452,21 @@ gsql_make_field_array (PGresult *result)
 
   if (numfields > 0)
     {
-      gr->fields = xmalloc ((numfields + 1) * sizeof (gsql_field *));
+      gr->fields = (gsql_field **)xmalloc
+	((numfields + 1) * sizeof (gsql_field *));
 
       for (i = 0; i < numfields; i++)
 	{
 	  gfield = (gsql_field *)xmalloc (sizeof (gsql_field));
 	  initialize_gsql_field (gfield);
 
-	  /* FIXME: I'm not sure this this correct
-	     ... gsql_make_field_array, in it's MySQL incarnation,
-	     seemed to only fill in one row.  So ... I've done the
-	     same here, getting only the first row.  But if it's a
-	     cursor ... well, then I need to do surgery to get that
-	     bullet out of my foot. :) */
-
 	  /* Pgres says we have to then copy this value out ... */
-	  gfield->value = strdup (PQgetvalue (result, 0, i));
+	  gfield->name = strdup (PQfname (result, i));
+	  gfield->fsize = (int)PQfsize (result, i);
+	  gfield->ftype = (int)PQftype (result, i);
+	  gfield->nullable = (int)PQgetisnull (result, 0, i);
+	  gfield->value = PQgetvalue (result, 0, i);
+	  if (gfield->value) gfield->value = strdup (gfield->value);
 	  gr->fields[i] = gfield;
 	}
 
@@ -414,9 +501,26 @@ gsql_db_list_tables (Database *db,
 		     char *table_name,
 		     char *table_type)
 {
+  static char query_string[1024] = { '\0' };
   gsql_result *gr = (gsql_result *) NULL;
-  PGresult *mr = PQexec (db->sock, "SELECT relname from pg_class where relkind='r' and relname!~'^pg' and relname!~'^Inv';");
-  ExecStatusType status = PQresultStatus (mr);
+  ExecStatusType status = PGRES_FATAL_ERROR;
+  PGresult *mr;
+  int inhibit = empty_string_p (pagefunc_get_variable
+				("pgsql::allow-system-tables"));
+
+  query_string[0] = '\0';
+  strcat (query_string, "SELECT relname FROM pg_class ");
+  strcat (query_string, "WHERE relkind = 'r' ");
+  if (inhibit)
+    strcat (query_string, "AND relname NOT LIKE 'pg_%' ");
+  strcat (query_string, "AND relname NOT LIKE 'Inv%' ");
+  strcat (query_string, "ORDER BY relname");
+
+  pagefunc_set_variable ("pgsql::recent-query", query_string);
+
+  mr = PQexec (db->sock, query_string);
+  if (mr != (PGresult *)NULL)
+    status = PQresultStatus (mr);
 
   /* Read stuff in if we got anything from our query. */
   if (status == PGRES_TUPLES_OK)
@@ -433,11 +537,9 @@ gsql_db_list_tables (Database *db,
  *
  ****************************************************************/
 
-/* FIXME: wow, are these ever broken. :) */
-
-#define gsql_field_name(f) (f->value)
-#define gsql_field_table(f) (f->value)
-#define gsql_field_length(f) (strlen(f->value))
+#define gsql_field_name(f) (f->name)
+#define gsql_field_table(f) (f->table)
+#define gsql_field_length(f) (f->fsize)
 #define gsql_field_is_primary_key(f) (0)
 #define gsql_field_is_unique(f) (0)
 #define gsql_field_is_not_null(f) (1)
@@ -445,42 +547,121 @@ gsql_db_list_tables (Database *db,
    /* Future ODBC compatibility */
 #define gsql_field_qualifier(f) (f->qualifier)
 #define gsql_field_owner(f) (f->owner)
-#define gsql_field_typename(f) (f->typename)
+#define gsql_field_type_name(f) (f->type_name)
 #define gsql_field_precision(f) (f->precision)
 #define gsql_field_scale(f) (f->scale)
 #define gsql_field_radix(f) (f->radix)
 
+#define FIELD_TYPE_BOOL         16
+#define FIELD_TYPE_BYTEA        17
+#define FIELD_TYPE_CHAR         18
+#define FIELD_TYPE_NAME         19
+#define FIELD_TYPE_CHAR16       20
+#define FIELD_TYPE_INT2         21
+#define FIELD_TYPE_INT28        22
+#define FIELD_TYPE_INT4         23
+#define FIELD_TYPE_REGPROC      24
+#define FIELD_TYPE_TEXT         25
+#define FIELD_TYPE_OID          26
+#define FIELD_TYPE_TID          27
+#define FIELD_TYPE_XID          28
+#define FIELD_TYPE_CID          29
+#define FIELD_TYPE_OID8         30
+#define FIELD_TYPE_SET          32
+#define FIELD_TYPE_CHAR2       409
+#define FIELD_TYPE_CHAR4       410
+#define FIELD_TYPE_CHAR8       411
+#define FIELD_TYPE_POINT       600
+#define FIELD_TYPE_LSEG        601
+#define FIELD_TYPE_PATH        602
+#define FIELD_TYPE_BOX         603
+#define FIELD_TYPE_POLYGON     604
+#define FIELD_TYPE_FILENAME    605
+#define FIELD_TYPE_FLOAT4      700
+#define FIELD_TYPE_FLOAT8      701
+#define FIELD_TYPE_ABSTIME     702
+#define FIELD_TYPE_RELTIME     703
+#define FIELD_TYPE_TINTERVAL   704
+#define FIELD_TYPE_UNKNOWN     705
+#define FIELD_TYPE_MONEY       790
+#define FIELD_TYPE_OIDINT2     810
+#define FIELD_TYPE_OIDINT4     910
+#define FIELD_TYPE_OIDNAME     911
+#define FIELD_TYPE_BPCHAR     1042
+#define FIELD_TYPE_VARCHAR    1043
+#define FIELD_TYPE_DATE       1082
+#define FIELD_TYPE_TIME       1083
+
 static int
 gsql_field_type (gsql_field *field)
 {
-  /* FIXME: somebody shoot me for this. */
-  return GSQL_CHAR;
+  int result = 0;
 
-  /*
-    switch (1)
+  /* FIXME: somebody shoot me for this. (andy) */
+  /* Oh, don't worry, I will (bfox) */
+  /* return GSQL_CHAR; */
+
+  switch (field->ftype)
     {
-    case FIELD_TYPE_CHAR:	return (GSQL_CHAR);
-    case FIELD_TYPE_NUMERIC:	return (GSQL_NUMERIC);
-    case FIELD_TYPE_DECIMAL:	return (GSQL_DECIMAL);
-    case FIELD_TYPE_INT24:	return (GSQL_INTEGER);
-    case FIELD_TYPE_SMALLINT:	return (GSQL_SMALLINT);
-    case FIELD_TYPE_FLOAT:	return (GSQL_FLOAT);
-    case FIELD_TYPE_REAL:	return (GSQL_REAL);
-    case FIELD_TYPE_DOUBLE:	return (GSQL_DOUBLE);
-    case FIELD_TYPE_STRING:     return (GSQL_VARCHAR); */
+    case FIELD_TYPE_BYTEA:
+    case FIELD_TYPE_BOOL:	result = GSQL_BOOLEAN;	break;
 
-      /* non-standard ANSI follows */
-  /*    case FIELD_TYPE_NULL:     return (GSQL_NULL);
-	case FIELD_TYPE_TINY_BLOB:     return (GSQL_TINY_BLOB);
-	case FIELD_TYPE_MEDIUM_BLOB:     return (GSQL_MEDIUM_BLOB);
-	case FIELD_TYPE_LONG_BLOB:     return (GSQL_LONG_BLOB);
-	case FIELD_TYPE_BLOB:     return (GSQL_BLOB);
-	case FIELD_TYPE_STRING:     return (GSQL_STRING);
-	case FIELD_TYPE_VAR_STRING:     return (GSQL_VAR_STRING);
+    case FIELD_TYPE_NAME:	result = GSQL_CHAR;	break;
+    case FIELD_TYPE_CHAR2:	result = GSQL_CHAR;	break;
+    case FIELD_TYPE_CHAR4:	result = GSQL_CHAR;	break;
+    case FIELD_TYPE_CHAR8:	result = GSQL_CHAR;	break;
+    case FIELD_TYPE_CHAR16:	result = GSQL_CHAR;	break;
+    case FIELD_TYPE_CHAR:	result = GSQL_CHAR;	break;
 
-	default:		return (GSQL_ANY);
-	}
-  */
+    case FIELD_TYPE_INT2:	result = GSQL_SMALLINT;	break;
+
+    case FIELD_TYPE_INT28:	result = GSQL_INTEGER;	break;
+    case FIELD_TYPE_INT4:	result = GSQL_INTEGER;	break;
+
+
+      /* Variable length character fields. */
+    case FIELD_TYPE_VARCHAR:	result = GSQL_VARCHAR;	break;
+    case FIELD_TYPE_BPCHAR:	result = GSQL_VARCHAR;	break;
+    case FIELD_TYPE_FILENAME:	result = GSQL_VARCHAR;	break;
+    case FIELD_TYPE_PATH:	result = GSQL_VARCHAR;	break;
+    case FIELD_TYPE_TEXT:	result = GSQL_VARCHAR;	break;
+    case FIELD_TYPE_OIDNAME:	result = GSQL_VARCHAR;	break;
+
+      /* Object Identitifiers are integers, right? */
+    case FIELD_TYPE_OID:	result = GSQL_INTEGER;	break;
+    case FIELD_TYPE_TID:	result = GSQL_INTEGER;	break;
+    case FIELD_TYPE_XID:	result = GSQL_INTEGER;	break;
+    case FIELD_TYPE_CID:	result = GSQL_INTEGER;	break;
+    case FIELD_TYPE_OID8:	result = GSQL_INTEGER;	break;
+
+    case FIELD_TYPE_OIDINT2:	result = GSQL_INTEGER;	break;
+    case FIELD_TYPE_OIDINT4:	result = GSQL_INTEGER;	break;
+
+
+      /* As are types related to seconds since epoch, etc. */
+    case FIELD_TYPE_ABSTIME:	result = GSQL_INTEGER;	break;
+    case FIELD_TYPE_RELTIME:	result = GSQL_INTEGER;	break;
+    case FIELD_TYPE_TINTERVAL:	result = GSQL_INTEGER;	break;
+    case FIELD_TYPE_TIME:	result = GSQL_INTEGER;	break;
+
+    case FIELD_TYPE_MONEY:	result = GSQL_MONEY;	break;
+    case FIELD_TYPE_DATE:	result = GSQL_DATE;	break;
+
+    case FIELD_TYPE_FLOAT4:	result = GSQL_REAL;	break;
+    case FIELD_TYPE_FLOAT8:	result = GSQL_DOUBLE;	break;
+
+    case FIELD_TYPE_SET:	result = GSQL_ANY;	break;
+    case FIELD_TYPE_POINT:	result = GSQL_ANY;	break;
+    case FIELD_TYPE_LSEG:	result = GSQL_ANY;	break;
+    case FIELD_TYPE_BOX:	result = GSQL_ANY;	break;
+    case FIELD_TYPE_POLYGON:	result = GSQL_ANY;	break;
+    case FIELD_TYPE_REGPROC:	result = GSQL_ANY;	break;
+
+    case FIELD_TYPE_UNKNOWN:	result = GSQL_ANY;	break;
+
+    default:			result = GSQL_ANY;
+    }
+  return (result);
 }
 
 /* This needs to build the array of gsql_fields. */
@@ -489,20 +670,24 @@ gsql_field_type (gsql_field *field)
 static gsql_result *
 gsql_list_fields (Database *db, char *tablename)
 {
-  static char pq_column_lookup[256];
+  static char query_string[256];
+  gsql_result *gr = (gsql_result *)NULL;
   PGresult *mr;
+  int status = PGRES_FATAL_ERROR;
 
-  /* This cannot work.  If the table is created, and has no entries,
-     then the result of the query is empty. */
-  sprintf (pq_column_lookup, "SELECT * FROM %s;\n", tablename)
-  mr = PQexec (db->sock, selectstring);
+  sprintf (query_string, "SELECT * FROM %s;\n", tablename);
+  pagefunc_set_variable ("pgsql::recent-query", query_string);
+  mr = PQexec (db->sock, query_string);
 
-  if (PQresultStatus (mr) == PGRES_TUPLES_OK)
+  if (mr != (PGresult *)NULL)
+    status = PQresultStatus (mr);
+
+  if (status == PGRES_TUPLES_OK)
     {
       register int i;
       int numfields = PQnfields (mr);
-      gsql_result *gr = (gsql_result *)xmalloc (sizeof (gsql_result));
 
+      gr = (gsql_result *)xmalloc (sizeof (gsql_result));
       gr->fields = (gsql_field **)xmalloc
 	((numfields + 1) * sizeof (gsql_field *));
 
@@ -510,20 +695,19 @@ gsql_list_fields (Database *db, char *tablename)
 	{
 	  gsql_field *gfield = (gsql_field *)xmalloc (sizeof (gsql_field));
 	  initialize_gsql_field (gfield);
-	  gfield->value = strdup (PQfname (mr, i));
-	  gfield->
+	  gfield->name = strdup (PQfname (mr, i));
+	  /* gfield->nullable = PQgetisnull (mr, 0, i); */
+	  gfield->table = strdup (tablename);
+	  gfield->fsize = (int)PQfsize (mr, i);
+	  gfield->ftype = (int)PQftype (mr, i);
 	  gr->fields[i] = gfield;
 	}
       gr->fields[i] = (gsql_field *)NULL;
     }
-  if (mr != (PGresult *)NULL)
-    {
-      PQclear(mr);
-      mr = (PGresult *)NULL;
-    }
 
-  xfree(selectstring);
-  return gr;
+  gr->postgres_result = mr;
+
+  return (gr);
 }
 
 static int
@@ -558,7 +742,6 @@ gsql_get_column_table_name (gsql_result *gr)
 
 /* FIXME: this postgres module doesn't do authentication yet.
    No password, UID stuff. */
-
 static void
 gsql_connect (char *dsn, Database *db)
 {
@@ -566,18 +749,13 @@ gsql_connect (char *dsn, Database *db)
   char *dbhost = dsn_lookup ("host", dsn);
   char *dbname = dsn_lookup ("database", dsn);
 
-  /*
-    char *user = dsn_lookup ("uid", dsn);
-    char *pass = dsn_lookup ("pwd", dsn);
-  */
-
   db->connected = 0;
   db->dbname = dbname;
   db->hostname = dbhost;
 
   if ((dbhost != (char *) NULL) && (dbname != (char *)NULL))
     {
-      sock = PQsetdb (dbhost, "5432", NULL, "/dev/null", dbname);
+      sock = PQsetdb (dbhost, NULL, NULL, "/dev/null", dbname);
 
       db->sock = sock;
       if (PQstatus (sock) == CONNECTION_BAD)
@@ -601,10 +779,6 @@ gsql_connect (char *dsn, Database *db)
       gsql_save_error_message (db, e->buffer);
       bprintf_free_buffer (e);
     }
-  /*
-    xfree (user);
-    xfree (pass);
-  */
 }
 
 static void
@@ -637,12 +811,20 @@ pf_host_databases (PFunArgs)
       host = strdup ("localhost");
     }
 
-  sock = PQsetdb (host, "5432", NULL, "/dev/null", "template1");
-  if (PQstatus (sock) != CONNECTION_BAD)
+  sock = PQsetdb (host, NULL, NULL, my_pgtty(), "template1");
+
+  if (PQstatus (sock) == CONNECTION_OK)
     {
-      PGresult *result = PQexec (sock, "SELECT datname FROM pg_database;");
-      int nrows = PQntuples (result);
-      ExecStatusType status = PQresultStatus (result);
+      char *query_string = "SELECT datname FROM pg_database";
+      PGresult *result;
+      ExecStatusType status;
+      int nrows;
+
+      pagefunc_set_variable ("pgsql::recent-query", query_string);
+      
+      result = PQexec (sock, query_string);
+      status = result ? PQresultStatus (result) : PGRES_FATAL_ERROR;
+      nrows = result ? PQntuples (result) : 0;
 
       if (nrows != 0 && ((status == PGRES_TUPLES_OK) ||
 			 (status == PGRES_COMMAND_OK)))
