@@ -55,8 +55,15 @@ static PFunDesc func_table[] =
   { (char *)NULL,	0, 0, (PFunHandler *)NULL }
 };
 
-DOC_SECTION (RELATIONAL-OPERATORS)
 PACKAGE_INITIALIZER (initialize_relational_functions)
+DEFINE_SECTION (RELATIONAL-OPERATORS, boolean; relational; logical, 
+"<i>Relational operators</i> in <Meta-HTML> return information about the
+relationship between two items.  There are three logical operators,
+<funref RELATIONAL-OPERATORS and>, <funref RELATIONAL-OPERATORS
+or>,  and <funref RELATIONAL-OPERATORS not>.
+
+There are several other operators which compare numeric values; the
+section <secref ARITHMETIC-OPERATORS> cover those in detail.", "")
 
 DEFUN (pf_not, &optional body,
 "<var body> is evaluated.  If the result is the empty string, then the
@@ -80,7 +87,7 @@ in it.  Thus, in typical usage one might write:
   if (sexp) free (sexp);
 }
 
-DEFUN (pf_and, &rest expr...,
+DEFUN (pf_and, &unevalled &rest expr...,
 "<code>and</code> evaluates each <var expr> given until one of them
 evaluates to the empty string, or until they are all exhausted.  The
 result is the result of the last evaluation.  Evaluating just <example
@@ -95,6 +102,19 @@ Examples:
 <and this <get-var foo>> -->
 <set-var foo=bar>
 <and this long list <get-var foo>> --> bar
+</example>
+
+<tag and> could have been defined in <Meta-HTML> as follows:
+<example>
+<define-tag and &unevalled &rest expressions[] whitespace=delete>
+  <set-var result=true>
+  <foreach :expr expressions>
+    <set-var result = <get-var :expr>>
+    <if <not <get-var-once result>>
+      <return>>
+  </foreach>
+  <get-var-once result>
+</define-tag>
 </example>")
 {
   register int i = 0;
@@ -126,7 +146,7 @@ Examples:
     }
 }
 
-DEFUN (pf_or, &rest expr...,
+DEFUN (pf_or, &unevalled &rest expr...,
 "<code>or</code> evaluates each <var expr> given until one of them
 evaluates to a non-empty string, or until they are all exhausted.  The
 result is the result of the last evaluation.  Evaluating just <example
@@ -141,6 +161,18 @@ Examples:
 <or this <get-var foo>> --> this
 <set-var foo=bar>
 <or <get-var foo> this> --> bar
+</example>
+
+<tag or> could have been defined in <Meta-HTML> as follows:
+<example>
+<define-tag or &unevalled &rest expressions[] whitespace=delete>
+  <foreach :expr expressions>
+    <set-var result = <get-var :expr>>
+    <if <get-var-once result>
+        <break>>
+  </foreach>
+  <get-var-once result>
+</define-tag>
 </example>")
 {
   register int i = 0;
