@@ -89,7 +89,96 @@ static PFunDesc func_table[] =
 };
 
 PACKAGE_INITIALIZER (initialize_session_functions)
-DOC_SECTION (SESSION-OPERATORS)
+DEFINE_SECTION (SESSION-OPERATORS, ,
+"<Meta-HTML> solves the problem of keeping state for a particular
+session, and it does it in a server and browser independent
+manner. Our method requires no additional CGI programs to be written,
+and you don't have to be a programmer to take advantage of it.
+
+The basic essence of sessions is the <i>session database</i>.  An
+entry in the session database can have exactly one of three possible
+states: <i>present</i>, <i>missing</i>, or <i>timed out</i>.",
+
+"Using the session commands, you can:
+
+<ul>
+<li>
+<b>Create a session.</b><br> This generates a unique entry
+in the session database, and returns that information as a session
+identifier (<strong>SID</strong>).  You can associate a special
+<i>key</i> with the session at creation time, and you can decide
+whether the key must be unique or not.  Session information can be
+located by either the SID or the key.
+
+<p><li> <b>Delete a session.</b><br>
+This removes the entry in the database associated with a specific SID,
+thus placing the session in the \"missing\" state.
+
+<p><li> <b>Change a session's timeout value.</b><br>
+With each session there is an associated <i>timeout</i> value, which
+is the number of minutes the session can live without any activity on
+the part of the user.  When the session is created, a default timeout
+is installed; currently this is five minutes.  Using
+<funref SESSION-OPERATORS >set-session-timeout>, you can change this
+value. A special signal value of \"<code>0</code>\" causes the session
+to live forever (or until it is deleted with <funref SESSION-OPERATORS
+delete-session>).
+
+<p><li> <b>Require a session to be present.</b><br>
+Using the <funref session-operators require-session> form allows you
+to check the state of the session, and optionally execute code for the
+cases where it is missing or timed out.
+
+<p><li> <b>Manipulate persistent variables.</b><br>
+Commands exist to associate a name with a value in the session
+database, to retrieve the value associated with a name in the session
+database, and to store and retrieve a group of associations at once
+from the session database.
+</ul>")
+
+DEFVAR (SID, "The current session ID, as set by the function
+<funref session-operators session::initialize>.
+
+The session ID is passed to and from the browser by either the use of
+HTTP Cookies, or in the URL, as the first element of the path.
+
+Normally, you never need to reference this variable directly, since
+the various internal functions which deal with sessions know how to
+set, get, and/or modify it without your help.
+
+Related functions: <funref Session-Operators create-session>,
+<funref Session-Operators delete-session>, <funref Session-Operators
+set-session-var>.")
+
+DEFVAR (mhttpd::session-database-file, "The fully qualified pathname to the
+current session database.  Setting this variable only has meaning in the
+<code>mhttpd.conf</code> or <code>engine.conf</code> files.
+
+If this variable is not set, then the default value of
+<code>\"/tmp/sessions.db\"</code> is used.
+
+If the server or engine cannot access this file, no session state
+information may be saved or retrieved.")
+
+DEFVAR (MHTML::SID-PREFIX, "When this variable is present, the
+<code>PATH</code> component of a returned <code>Set-Cookie</code> MIME
+header is set to its value.  Normally, the value of <varref
+MHTML::RELATIVE-PREFIX> is used.
+
+If you need to use <code>MHTML::SID-PREFIX</code>, you probably have a
+very complex directory structure, and the need for every page access
+to have a cookie associated with it.  In this case, setting
+<code>MHTML::SID-PREFIX</code> will have the effect of flattening your
+directory structure in the browser's view.
+
+Generally, correctly written <meta-html> applications do not need to
+use this variable, and it might be used in only the most complicated
+of Web sites, with multiple sessions per connectee, etc.
+
+The explanation of HTTP Cookies is beyond the scope of this document.
+Reference information on Cookies can be found through <a
+href=\"http://home.netscape.com/newsref/std/cookie_spec.html\"> this
+link.</a>")
 
 /* Get the values of VAR1 .. VARn and concatenate them to make a key.
    Create the session with that key.
