@@ -43,6 +43,7 @@ static void pf_stream_get_contents (PFunArgs);
 static void pf_stream_readable (PFunArgs);
 static void pf_stream_writable (PFunArgs);
 static void pf_stream_shutdown (PFunArgs);
+static void pf_stream_info (PFunArgs);
 
 static PFunDesc func_table[] =
 {
@@ -54,12 +55,34 @@ static PFunDesc func_table[] =
   { "STREAM-READABLE",		0, 0, pf_stream_readable },
   { "STREAM-WRITABLE",		0, 0, pf_stream_writable },
   { "STREAM-SHUTDOWN",		0, 0, pf_stream_shutdown },
+  { "STREAM-INFO",		0, 0, pf_stream_info },
   { (char *)NULL,		0, 0, (PFunHandler *)NULL }
 };
 
-DOC_SECTION (STREAM-OPERATORS)
 PACKAGE_INITIALIZER (initialize_stream_functions)
 
+DEFINE_SECTION (STREAM-OPERATORS, files; streams; reading; writing;
+creating, "A <i>stream</i> is a data object upon which various input
+and output operations may be performed.
+
+<meta-html> provides commands for creating, opening, reading from, and
+writing to, stream objects.  The referenced underlying object may
+either be a file or a network connection.", "There are three special
+names which can be used with <funref stream-operators
+with-open-stream>, these are <code>*standard-input*</code>,
+<code>*standard-output*</code>, and <code>*standard-error*</code>.
+
+<code>*standard-input*</code> always names the stream that from which
+input was being read at the time the engine, server, or
+<code>mhc</code> invoked.  <code>*standard-output*</code> always names
+the stream to which output is being written by the engine, server or
+<code>mhc</code>.
+
+<code>*standard-error*</code> always names the stream where error
+output will appear -- this stream could be the same as the one named
+by <code>*standard-output*</code>, but this does not have to be the
+case.")
+
 typedef enum
   { stream_FILE, stream_TCP, stream_PROG, stream_CLOSED }
 StreamType;
@@ -539,6 +562,96 @@ get_stream_reference (Package *vars)
   return (stream);
 }
 
+static char *
+stream_type_string (StreamType type)
+{
+  switch (type)
+    {
+    case stream_FILE: return ("FILE");
+    case stream_TCP:  return ("TCP");
+    case stream_PROG: return ("PROG");
+    default:	      return ("CLOSED");
+    }
+
+  /* NOT REACHED */
+  return ((char *)NULL);
+}
+
+static char *
+stream_mode_string (int mode)
+{
+  if (O_RDONLY == mode)
+    return ("READ");
+  else if (O_WRONLY == mode)
+    return ("WRITE");
+  else if (O_RDWR == mode)
+    return ("READ-WRITE");
+  else if ((O_APPEND | O_WRONLY | O_CREAT) == mode)
+    return ("APPEND");
+  else if ((O_TRUNC | O_RDWR | O_CREAT) == mode)
+    return ("WRITE-CREATE");
+  else
+    return ("READER?");
+}
+      
+/* Return an alist representing the stream referenced by the first
+   positional argument in VARS. */
+char *
+mhtml_stream_reference_to_alist (Package *vars)
+{
+  Stream *stream = get_stream_reference (vars);
+  char *result = (char *)NULL;
+
+  if (stream != (Stream *)NULL)
+    {
+      Package *p = symbol_get_package ((char *)NULL);
+
+      forms_set_tag_value_in_package
+	(p, "name", stream->name ? stream->name : "Anonymous Stream");
+
+      forms_set_tag_value_in_package
+	(p, "type", stream_type_string (stream->type));
+      mhtml_set_numeric_variable_in_package
+	(p, "descriptor",
+	 (stream->type == stream_PROG) ? stream->ifd : stream->fd);
+      forms_set_tag_value_in_package
+	(p, "mode", stream_mode_string (stream->mode));
+      mhtml_set_numeric_variable_in_package
+	(p, "timeout", stream->timeout);
+
+      /* Handle the flags. */
+      {
+	Symbol *sym = symbol_intern_in_package (p, "flags");
+
+	if (stream->flags & stream_MUST_SEEK)
+	  symbol_add_value (sym, "MUST-SEEK");
+	if (stream->flags & stream_NOTIMEOUT)
+	  symbol_add_value (sym, "NOTIMEOUT");
+      }
+
+      if (stream->type == stream_PROG)
+	mhtml_set_numeric_variable_in_package (p, "pid", (int)stream->pid);
+
+      result = package_to_alist (p, 1);
+    }
+
+  return (result);
+}
+
+DEFUN (pf_stream_info, stream,
+       "Return an association list providing information about <var stream>.")
+{
+  char *alist = mhtml_stream_reference_to_alist (vars);
+
+  if (alist != (char *)NULL)
+    {
+      int len = strlen (alist);
+      bprintf_insert (page, start, "%s", alist);
+      free (alist);
+      *newstart += len;
+    }
+}
+
 /* Gets called if/when we run out of time waiting for a known number
    of bytes that are available. */
 static int operation_timed_out_p = 0;
