@@ -39,6 +39,9 @@ extern "C"
 extern int mhtml_stdin_fileno;
 extern int mhtml_stdout_fileno;
 extern int mhtml_stderr_fileno;
+/* Return an alist representing the stream referenced by the first
+   positional argument in VARS. */
+extern char *mhtml_stream_reference_to_alist (Package *vars);
 
 #if defined (__cplusplus)
 }
