@@ -58,6 +58,7 @@ static void pf_string_neq (PFunArgs);
 static void pf_plain_text (PFunArgs);
 static void pf_base64decode (PFunArgs);
 static void pf_base64encode (PFunArgs);
+static void pf_char_offsets (PFunArgs);
 
 #if defined (HAVE_CRYPT)
 #  if defined (HAVE_CRYPT_H)
@@ -85,6 +86,7 @@ static PFunDesc func_table[] =
   { "PLAIN-TEXT",	1, 0, pf_plain_text },
   { "BASE64DECODE",	0, 0, pf_base64decode },
   { "BASE64ENCODE",	0, 0, pf_base64encode },
+  { "CHAR-OFFSETS",	0, 0, pf_char_offsets },
 #if defined (HAVE_CRYPT)
   { "UNIX::CRYPT",	0, 0, pf_unix_crypt },
 #endif
@@ -92,14 +94,20 @@ static PFunDesc func_table[] =
   { (char *)NULL,	0, 0, (PFunHandler *)NULL }
 };
 
-DOC_SECTION (STRING-OPERATORS)
 PACKAGE_INITIALIZER (initialize_string_functions)
+DEFINE_SECTION (STRING-OPERATORS, strings; characters; changing case,
+"There is a single function in <meta-html> which performs pattern
+matching, substring extraction, and substring deletion.  For
+convenience, a blind substring extraction function is supplied as
+well.  Three functions perform the three most common case changes.
+Finally, the <funref \"string operators\" pad> function allows
+alignment of fixed-width text.  ", "")
 
 DEFUN (pf_string_length, string,
 "Returns the number of characters present in <var string>.
 
 <complete-example>
-   <string-length \"This is an interesting string\">
+<string-length \"This is an interesting string\">
 </complete-example>")
 {
   char *string = mhtml_evaluate_string (get_positional_arg (vars, 0));
@@ -301,8 +309,8 @@ Here are a few examples:
 
 DEFUN (pf_substring, string &key start end,
 "Extracts the substring of <var string> whose first character starts
-at offset <var start>, and whose last character ends at offset <var
-end>. The indexing is zero-based, so that:
+at offset <var start>, and whose last character ends at offset
+<var end>. The indexing is zero-based, so that:
 
 <example>
   <substring \"Hello\" 1 2> --> \"e\"
@@ -310,7 +318,17 @@ end>. The indexing is zero-based, so that:
 
 This function is useful when you know in advance which part of the
 string you would like to extract, and do not need the pattern matching
-facilities of <funref strings match>.")
+facilities of <funref string-operators match>.
+
+If you wish to index through each character of a string, the most direct
+way is to convert it to an array first using
+<funref string-operators string-to-array>, and then use the
+<funref array-operators foreach> function to iterate over the members.
+<complete-example>
+<set-var s=\"This is a string.\">
+<string-to-array <get-var-once s> chars>
+<foreach character chars><get-var character>-</foreach>
+</complete-example>")
 {
   char *str_arg = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *beg_arg = mhtml_evaluate_string (get_positional_arg (vars, 1));
@@ -1239,6 +1257,43 @@ strings into their original, pre-encoded state.
   xfree (string);
 }
 
+DEFUN (pf_char_offsets, string ch &key caseless,
+"Return an array of numbers,  each one representing the offset from the
+start of <var string> of <var ch>.  This function is useful for finding
+candidate locations for word-wrapping, for example.
+Here is a complete example:
+<complete-example>
+<char-offsets \"This is a list\" \" \">
+</complete-example>")
+{
+  char *string = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  char *ch = mhtml_evaluate_string (get_positional_arg (vars, 1));
+  char *caseless_arg = mhtml_evaluate_string (get_value (vars, "CASELESS"));
+  int caseless_p = !empty_string_p (caseless_arg);
+
+  if ((string != (char *)NULL) && (ch != (char *)NULL))
+    {
+      register int i;
+
+      for (i = 0; string[i] != '\0'; i++)
+	{
+	  if ((string[i] == *ch) ||
+	      (caseless_p && (tolower (string[i]) == tolower (*ch))))
+	    {
+	      static char digits[40];
+	      sprintf (digits, "%d", i);
+	      bprintf_insert (page, start, "%s\n", digits);
+	      start += 1 + strlen (digits);
+	    }
+	}
+    }
+
+  *newstart = start;
+  xfree (string);
+  xfree (ch);
+  xfree (caseless_arg);
+}
+
 #if defined (HAVE_CRYPT)
 /* Create a password from cleartext. */
 static char *
@@ -1307,6 +1362,7 @@ pf_unix_crypt (PFunArgs)
 
   if (!empty_string_p (saltarg))
     {
+      salt[1] = '\0';
       salt[0] = saltarg[0];
       if (saltarg[1]) salt[1] = saltarg[1];
     }
