@@ -47,6 +47,28 @@
 #  include <mac_port.h>
 #endif
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
+/* Boy, I hate to do this, but I have to. */
+#if defined (METAHTML_PROFILER) /* Symbol used because it is true for Mhtml. */
+
+typedef struct
+{
+  int type;		/* Either user_MACRO, user_SUBST, or user_DEFUN. */
+  int flags;		/* Interesting bits about this function. */
+  int debug_level;	/* How much debugging to do. */
+  char *name;		/* The name of this macro, function, or subst. */
+  char *body;		/* The body of this macro function, or subst. */
+  char *packname;	/* Default package for the scope of this function. */
+  char **named_parameters; /* Variables to bind during function invocation. */
+  char **documentation;	/* The first set of comments that follow the defun. */
+  void *profile_info;
+} UserFunction;
+#endif
+
 /* The list of all created packages. */
 Package **AllPackages = (Package **)NULL;
 static int AP_index = 0;
@@ -192,19 +214,19 @@ static SymbolTable *
 symbol_create_table (int rows)
 {
   register int i;
-  SymbolTable *new = (SymbolTable *)xmalloc (sizeof (SymbolTable));
+  SymbolTable *newtab = (SymbolTable *)xmalloc (sizeof (SymbolTable));
 
   if (rows == 0)
     rows = symbol_small_prime;
 
-  new->symbol_list = (SymbolList **)xmalloc (rows * sizeof (SymbolList *));
-  new->rows = rows;
-  new->entries = 0;
+  newtab->symbol_list = (SymbolList **)xmalloc (rows * sizeof (SymbolList *));
+  newtab->rows = rows;
+  newtab->entries = 0;
 
   for (i = 0; i < rows; i++)
-    new->symbol_list[i] = (SymbolList *)NULL;
+    newtab->symbol_list[i] = (SymbolList *)NULL;
 
-  return (new);
+  return (newtab);
 }
 
 static int
@@ -411,11 +433,11 @@ symbol_destroy_package (Package *package)
 	      while (list)
 		{
 		  Symbol *sym = list->symbol;
-		  SymbolList *this = list;
+		  SymbolList *thissym = list;
 		  list = list->next;
 
 		  symbol_free (sym);
-		  free (this);
+		  free (thissym);
 		}
 
 	    }
@@ -540,7 +562,7 @@ symbol_intern_in_package (Package *package, char *name)
       SymbolTable *table = package->table;
       int row = symbol_hash (table, name);
       SymbolList *list = table->symbol_list[row];
-      SymbolList *new = (SymbolList *)xmalloc (sizeof (SymbolList));
+      SymbolList *newlist = (SymbolList *)xmalloc (sizeof (SymbolList));
 
       symbol = (Symbol *)xmalloc (sizeof (Symbol));
       memset (symbol, 0, sizeof (Symbol));
@@ -560,9 +582,9 @@ symbol_intern_in_package (Package *package, char *name)
       symbol->package = (void *)package;
       symbol->notifier = (int *)NULL;
 
-      new->symbol = symbol;
-      new->next = list;
-      table->symbol_list[row] = new;
+      newlist->symbol = symbol;
+      newlist->next = list;
+      table->symbol_list[row] = newlist;
       table->entries++;
     }
 
@@ -743,8 +765,8 @@ symbols_of_package (Package *package)
 }
 
 /* Free the individual pointers in ARRAY, and the ARRAY itself. */
-static void
-free_array (char **array)
+void
+symbol_free_array (char **array)
 {
   if (array != (char **)NULL)
     {
@@ -758,8 +780,8 @@ free_array (char **array)
 }
 
 /* Return the number of entries in ARRAY. */
-static int
-array_length (char **array)
+int
+symbol_array_length (char **array)
 {
   register int i = 0;
 
@@ -770,10 +792,10 @@ array_length (char **array)
 }
 
 /* Create and return a copy of ARRAY. */
-static char **
-copy_array (char **array)
+char **
+symbol_copy_array (char **array)
 {
-  int len = array_length (array);
+  int len = symbol_array_length (array);
   char **copy = (char **)NULL;
 
   if (len)
@@ -811,8 +833,8 @@ datablock_create (char *data, int length)
 Datablock *
 datablock_copy (Datablock *block)
 {
-  Datablock *new = datablock_create (block->data, block->length);
-  return (new);
+  Datablock *newblock = datablock_create (block->data, block->length);
+  return (newblock);
 }
 
 /* Free BLOCK and contents. */
@@ -843,7 +865,7 @@ symbol_free (Symbol *symbol)
 	case symtype_STRING:
 	  free (symbol->name);
 	  if (symbol->preserved_name) free (symbol->preserved_name);
-	  free_array (symbol->values);
+	  symbol_free_array (symbol->values);
 	  free (symbol);
 	  break;
 
@@ -867,7 +889,7 @@ symbol_reset (Symbol *symbol)
   switch (symbol->type)
     {
     case symtype_STRING:
-      free_array (symbol->values);
+      symbol_free_array (symbol->values);
       break;
 
     case symtype_BINARY:
@@ -941,14 +963,46 @@ symbol_copy (Symbol *symbol, Package *package)
   switch (symbol->type)
     {
     case symtype_STRING:
-      copy->values = copy_array (symbol->values);
+      copy->values = symbol_copy_array (symbol->values);
       copy->values_slots = copy->values_index;
       break;
 
     case symtype_FUNCTION:
+      copy->values = symbol->values;
+      break;
+
+#if defined (METAHTML_PROFILER)
+    case symtype_USERFUN:
+      {
+	UserFunction *newfun = (UserFunction *)xmalloc (sizeof (UserFunction));
+	UserFunction *old = (UserFunction *)symbol->values;
+	newfun->type = old->type;
+	newfun->flags= old->flags;
+	newfun->debug_level = 0;
+	newfun->name = old->name ? strdup (old->name) : (char *)NULL;
+	newfun->body = old->body ? strdup (old->body) : (char *)NULL;
+	newfun->packname =
+	  old->packname ? strdup (old->packname) : (char *)NULL;
+	if (old->named_parameters)
+	  newfun->named_parameters = symbol_copy_array (old->named_parameters);
+	else
+	  newfun->named_parameters = (char **)NULL;
+
+	if (old->documentation)
+	  newfun->documentation = symbol_copy_array (old->documentation);
+	else
+	  newfun->documentation = (char **)NULL;
+
+	newfun->profile_info = (void *)NULL;
+
+	copy->values = (char **)newfun;
+      }
+      break;
+#else
     case symtype_USERFUN:
       copy->values = symbol->values;
       break;
+#endif /* !METAHTML */
 
     case symtype_BINARY:
       copy->values = (char **)datablock_copy ((Datablock *)symbol->values);
@@ -1194,3 +1248,7 @@ main (int argc, char *argv[])
   return (0);
 }
 #endif /* TEST */
+
+#if defined (__cplusplus)
+}
+#endif
