@@ -81,7 +81,29 @@ static PFunDesc func_table[] =
 };
 
 PACKAGE_INITIALIZER (initialize_variable_functions)
-DOC_SECTION (VARIABLES)
+DEFINE_SECTION (VARIABLES, variables; symbols; lookup,
+"<Meta-HTML> provides a simple mechanism for the storage and retrieval
+of variables during a single (possibly recursive) processing pass.  In
+addition to this, functions are provided which test the value of a
+variable, and conditionally execute other code based upon the result.
+
+There is also a mechanism to group a set of variables using <secref
+packages>.",
+
+"<h3>Variable Types and Information</h3>
+
+There are internal functions in <meta-html> which create <i>binary</i>
+variables.  Such variables cannot be used in the ordinary way, since
+the data contained within them may not retain integrity once printed
+out and read back in.  You can check to see what the type of a
+particular variable is with the <funref variables symbol-info>
+function.
+
+Many variables are predefined by <Meta-HTML>, and made available to
+the page writer.  The <secref Page-Variables> section of this manual
+documents those variables fully, while the <secref server-variables>
+section contains more information on variables which control, or were
+specifically created by, the web server.")
 
 static void
 generic_set_variable (Package *vars, int debug_level, int eval_p, int readonly_p)
@@ -281,7 +303,7 @@ data, thus preventing further interpretation of the data.
 Example:
 <example>
 <set-var bar=HELLO>
-<set-var foo=\"<verbatim><get-var bar></verbatim>\">
+<set-var-verbatim foo=<get-var bar>>
 <get-var-once foo>   --> &lt;get-var bar&gt;
 </example>
 but...
@@ -605,14 +627,22 @@ DEFUN (pf_symbol_info, symbol,
 
 The information is two lines:
 
+<ul>
+<li> The type of the symbol, which is one of:
 <ol>
-<li> The type of the symbol, either <code>STRING</code>,
-<code>BINARY</code>, or <code>FUNCTION</code>.
-<li> The \"size\" of the symbol.
+<li><code>STRING</code>,
+<li><code>BINARY</code>,
+<li><code>USER DEFUN</code>,
+<li><code>USER SUBST</code>,
+<li><code>USER MACRO</code>,
+<li><code>PRIM DEFUN</code>, or
+<li><code>PRIM MACRO</code>.
 </ol>
+<li> The \"size\" of the symbol.
+</ul>
 
-For STRING variables, the size value is the number of elements in the
-array.<br>
+For <code>STRING</code>. variables, the size value is the number of
+elements in the array.<br>
 For BINARY variables, the size value is the number of bytes of binary
 data stored within.
 
@@ -633,7 +663,13 @@ The size value is zero for all other variable types.")
 	      break;
 
 	    case symtype_FUNCTION:
-	      bprintf_insert (page, start, "FUNCTION\n0");
+	      {
+		PFunDesc *desc = (PFunDesc *)sym->values;
+		if (desc->complexp)
+		  bprintf_insert (page, start, "PRIM MACRO\n0");
+		else
+		  bprintf_insert (page, start, "PRIM DEFUN\n0");
+	      }
 	      break;
 
 	    case symtype_BINARY:
@@ -646,13 +682,13 @@ The size value is zero for all other variable types.")
 		switch (((UserFunction *)sym->values)->type)
 		  {
 		  case user_MACRO:
-		    bprintf_insert (page, start, "USER MACRO");
+		    bprintf_insert (page, start, "USER MACRO\n0");
 		    break;
 		  case user_SUBST:
-		    bprintf_insert (page, start, "USER SUBST");
+		    bprintf_insert (page, start, "USER SUBST\n0");
 		    break;
 		  case user_DEFUN:
-		    bprintf_insert (page, start, "USER DEFUN");
+		    bprintf_insert (page, start, "USER DEFUN\n0");
 		    break;
 		  }
 	      }
@@ -730,7 +766,7 @@ Each <var to-var> becomes the repository of a copy of the information already st
 #if !defined (symtype_ARRAY)
 #define symtype_ARRAY 78
 #endif
-DEFUN (pf_coerce_var, varname &key type=(STRING|BINARY),
+DEFUN (pf_coerce_var, varname &key type=(STRING|BINARY|ARRAY),
 "Coerces <var varname>'s data to have the type specified by the
 argument to <var type>.  You can convert a binary object to a string
 object, and vice-versa.
