@@ -58,6 +58,11 @@
 #  include <pwd.h>
 #endif
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 #if !defined (errno)
 extern int errno;
 #endif
@@ -691,4 +696,6 @@ mhttpd_initialize_ssl (void)
 #endif
 }
 
-    
+#if defined (__cplusplus)
+}
+#endif
