@@ -75,6 +75,10 @@ extern "C"
 {
 #endif
 
+#if defined (__WINNT__)
+typedef RETSIGTYPE (*sig_t) (int);
+#endif /* __WINNT__ */
+
 #if !defined (errno)
 extern int errno;
 #endif
@@ -1201,7 +1205,8 @@ mhttpd_build_environment (HTTP_RESULT *result)
 
       for (i = 0; environ[i] != (char *)NULL; i++)
 	{
-	  if ((strncmp (environ[i], "HTTP", 4) == 0) &&
+	  if (((strncmp (environ[i], "HTTP", 4) == 0) ||
+	       (strncmp (environ[i], "REDIRECT", 8) == 0)) &&
 	      (strlen (environ[i]) < 255))
 	    {
 	      strcpy (var_and_val, environ[i]);
@@ -1897,7 +1902,7 @@ mhttpd_metahtml_engine (HTTP_RESULT *result)
       
       {
 	PAGE *p = page_create_page ();
-	bprintf (p, "<*parser*::push-file %s>", result->spec->physical_path);
+	bprintf (p, "<*parser*::push-file %s>", result->spec->requested_path);
 	page_process_page (p);
 	page_free_page (p);
       }
