@@ -33,6 +33,10 @@
 
 #include "result_codes.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
 typedef struct
 {
   char *tag;
@@ -111,4 +115,8 @@ extern void mhttpd_free_request (HTTP_REQUEST *request);
 extern char *fast_cgi_content;
 extern int fast_cgi_content_length;
 
+#if defined (__cplusplus)
+}
+#endif
+
 #endif /* !_HTTP_H_ */
