@@ -49,6 +49,11 @@
 #include "path_resolve.h"
 #include "globals.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 static DOC_SPEC *
 make_doc_spec (void)
 {
@@ -215,6 +220,16 @@ mhttpd_resolve_location (HTTP_REQUEST *request)
   temp = path_info;
   path_info = mhtml_path_translate (path_info);
   free (temp);
+  temp = path_info;
+
+  /* The following magic hack makes badly munged URLs work.  I don't like
+     it, but it is the path of least resistance. */
+  if ((temp[0] == '/') && (temp[1] == '%') && (temp[2] == '7') &&
+      ((temp[3] == 'e') || (temp[3] == 'E')))
+    {
+      temp[1] = '~';
+      memmove (temp + 2, temp + 4, strlen (temp + 3));
+    }
 
   /* Now do username lookup. */
   temp = mhtml_user_translate (path_info);
@@ -365,7 +380,7 @@ mhttpd_resolve_location (HTTP_REQUEST *request)
        4) Meta-HTML extension match. */
   {
     char **cgi_paths;
-    char *template;
+    char *the_template;
     int template_len;
     int executable_found = 0;
 
@@ -374,10 +389,10 @@ mhttpd_resolve_location (HTTP_REQUEST *request)
 
     for (i = 0; cgi_paths && cgi_paths[i]; i++)
       {
-	template = cgi_paths[i];
-	template_len = strlen (template);
+	the_template = cgi_paths[i];
+	template_len = strlen (the_template);
 
-	if ((strncmp (template, path_info, template_len) == 0) &&
+	if ((strncmp (the_template, path_info, template_len) == 0) &&
 	    ((path_info[template_len] == '\0') ||
 	     (path_info[template_len] == '/')))
 	  {
@@ -434,10 +449,10 @@ mhttpd_resolve_location (HTTP_REQUEST *request)
 
 	for (i = 0; cgi_paths && cgi_paths[i]; i++)
 	  {
-	    template = cgi_paths[i];
-	    template_len = strlen (template);
+	    the_template = cgi_paths[i];
+	    template_len = strlen (the_template);
 
-	    if (strncmp (template, path_info, template_len) == 0)
+	    if (strncmp (the_template, path_info, template_len) == 0)
 	      {
 		/* This prefix indicates that the next pathname component
 		   is an executable file.  Indicate that in the specification
@@ -560,3 +575,7 @@ mhttpd_resolve_location (HTTP_REQUEST *request)
 
   return (spec);
 }
+
+#if defined (__cplusplus)
+}
+#endif
