@@ -31,7 +31,15 @@
 #if !defined (_PATH_RESOLVE_H_)
 #define _PATH_RESOLVE_H_ 1
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
 extern DOC_SPEC *mhttpd_resolve_location (HTTP_REQUEST *request);
 extern void mhttpd_free_doc_spec (DOC_SPEC *spec);
 
+#if defined (__cplusplus)
+}
+#endif
+
 #endif /* !_PATH_RESOLVE_H_ */
