@@ -41,6 +41,7 @@
 #include <sys/types.h>
 #include <ctype.h>
 #include <xmalloc/xmalloc.h>
+
 #include "bprintf.h"
 
 /* The value which is printed when %s is called with a null pointer. */
@@ -533,6 +534,9 @@ bprintf_word_wrap (BPRINTF_BUFFER *buffer, int width)
   int col = 0;
   int last_space = 0;
 
+  /* Make sure that the last word is wrapped. */
+  bprintf (buffer, " ");
+
   for (i = 0; i < buffer->bindex; i++)
     {
       col++;
