@@ -31,6 +31,11 @@
 #if !defined (_BPRINTF_H_)
 #define _BPRINTF_H_
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 /* A structure used to hold onto a dynamically created buffer of text. */
 typedef struct {
   char *buffer;
@@ -77,4 +82,8 @@ extern int bprintf_insert_text (BPRINTF_BUFFER *buffer, int point, char *text);
 /* Word wrap the text in BUFFER making lines no longer than WIDTH. */
 extern void bprintf_word_wrap (BPRINTF_BUFFER *buffer, int width);
 
+#if defined (__cplusplus)
+}
+#endif
+
 #endif /* ! _BPRINTF_H_ */
