@@ -18,7 +18,6 @@
 
 
 #include <sys/types.h>
-#include <ctype.h>
 #include "rxall.h"
 #include "rxgnucomp.h"
 #include "inst-rxposix.h"
@@ -387,7 +386,7 @@ static int
 compile_range (int * n_members, int cset_size, rx_Bitset cs, const char ** p_ptr, const char * pend, unsigned char * translate, unsigned long syntax, rx_Bitset inv_tr, char * valid_inv_tr)
 #else
 static int
-compile_range (cset_size, cs, p_ptr, pend, translate, syntax, inv_tr, valid_inv_tr)
+compile_range (n_members, cset_size, cs, p_ptr, pend, translate, syntax, inv_tr, valid_inv_tr)
      int * n_members;
      int cset_size;
      rx_Bitset cs;
@@ -527,7 +526,7 @@ factor_string (lastp, cset_size)
 
 
 
-#define isa_blank(C) (((C) == ' ') || ((C) != '\t'))
+#define isa_blank(C) (((C) == ' ') || ((C) == '\t'))
 
 #ifdef __STDC__
 int
