@@ -47,7 +47,7 @@
 #include <arpa/inet.h>
 #endif
 
-#if !defined (HAVE_TYPE_SIG_T)
+#if !defined (HAVE_TYPE_SIG_T) || defined (__WINNT__)
 typedef void (*sig_t) (int);
 #endif
 
