@@ -50,7 +50,7 @@ static WispObject *read_database (char *filename);
 static WispObject *find_matches (WispObject *database, char *key, char *string);
 
 static char *rolodex_file = "/homes/UniAx/Admin/ROLODEX";
-static char *user_file_template = "/homes/%s/.phones";
+static char *user_file_template = "%s/.phones";
 
 int
 main (int argc, char *argv[])
@@ -74,9 +74,9 @@ main (int argc, char *argv[])
       if (entry)
 	{
 	  database_filename = (char *) xmalloc
-	    (strlen (user_file_template) + strlen (entry->pw_name) + 4);
+	    (strlen (user_file_template) + strlen (entry->pw_dir) + 4);
 
-	  sprintf (database_filename, user_file_template, entry->pw_name);
+	  sprintf (database_filename, user_file_template, entry->pw_dir);
 	}
     }
 
