@@ -86,8 +86,10 @@ static char *mdb_print (MDBArgs);
 static char *mdb_apropos (MDBArgs);
 static char *mdb_where (MDBArgs);
 static char *mdb_history (MDBArgs);
+static char *mdb_cd (MDBArgs);
 
-MDBCommand mdb_command_table[] = {
+MDBCommand mdb_command_table[] =
+{
   { "quit",	(char *)NULL,	mdb_quit,	"quit",
       "Quit using the debugger" },
 
@@ -106,6 +108,9 @@ MDBCommand mdb_command_table[] = {
   { "break",	"b",		mdb_breakpoint,	"break [FILENAME:]LINE",
       "Set a breakpoint for the RUN command" },
 
+  { "cd",	(char *)NULL,	mdb_cd,		"cd [DIRECTORY]",
+      "Change the current working directory, and mhtml::include-prefix" },
+
   { "delete",	"d",		mdb_delete,	"delete BREAKPOINT",
       "Delete an existing breakpoint" },
   
@@ -345,13 +350,14 @@ mdb_find_file (char *filename)
   MDB_File *file = (MDB_File *)NULL;
   char *incpref = pagefunc_get_variable ("mhtml::include-prefix");
   char *relpref = pagefunc_get_variable ("mhtml::relative-prefix");
+  char *wr = (char *)NULL;
   char *canon_name;
 
   if ((filename && incpref) &&
       (strncmp (filename, incpref, strlen (incpref)) == 0))
     canon_name = strdup (filename);
   else
-    canon_name = mhtml_canonicalize_file_name (filename, incpref, relpref);
+    canon_name = mhtml_canonicalize_file_name(filename, incpref, relpref, &wr);
 
   if (canon_name != (char *)NULL)
     {
@@ -488,17 +494,112 @@ mdb_redo (void)
   return (result);
 }
 
+static char **
+mdb_split_into_words (char *line)
+{
+  char **words = (char **)NULL;
+  int words_slots = 0;
+  int words_index = 0;
+
+  if (line != (char *)NULL)
+    {
+      register int done = 0;
+      register int start = 0;
+
+      while (!done)
+	{
+	  register int i;
+	  char *word;
+
+	  while (whitespace (line[start])) start++;
+
+	  if (line[start] == '\0')
+	    break;
+
+	  for (i = start; (line[i] != '\0') && !whitespace (line[i]); i++);
+	  word = (char *)xmalloc (1 + (i - start));
+	  strncpy (word, line + start, i - start);
+	  word[i - start] = '\0';
+
+	  start = i;
+
+	  if (words_index + 2 > words_slots)
+	    words = (char **)xrealloc
+	      (words, (words_slots += 5) * sizeof (char *));
+
+	  words[words_index++] = word;
+	  words[words_index] = (char *)NULL;
+	}
+    }
+  return (words);
+}
+
 static char *
 mdb_help (MDBArgs)
 {
   register int i;
   BPRINTF_BUFFER *buffer = bprintf_create_buffer ();
   char *result;
+  char **words = mdb_split_into_words (line);
 
-  for (i = 0; mdb_command_table[i].name; i++)
+  if (words != (char **)NULL)
     {
-      bprintf (buffer, "%25s    %s\n", mdb_command_table[i].invocation,
-	       mdb_command_table[i].description);
+      register int windex;
+
+      for (windex = 0; words[windex] != (char *)NULL; windex++)
+	{
+	  MDBCommand *command = find_command (words[windex]);
+
+	  if (command != (MDBCommand *)NULL)
+	    {
+	      bprintf (buffer, "%25s    %s\n",
+		       command->invocation, command->description);
+	    }
+	  else
+	    {
+	      BPRINTF_BUFFER *temp = bprintf_create_buffer ();
+	      char *fundoc;
+
+	      bprintf (temp, "<mdb::describe-function %s>", words[windex]);
+	      fundoc = mhtml_evaluate_string (temp->buffer);
+
+	      bprintf_free_buffer (temp);
+
+	      if (!empty_string_p (fundoc))
+		bprintf (buffer, "%s", fundoc);
+	      else
+		{
+		  xfree (fundoc);
+		  temp = bprintf_create_buffer ();
+		  bprintf (temp, "<mdb::function-documentation %s>",
+			   words[windex]);
+		  fundoc = mhtml_evaluate_string (temp->buffer);
+		  bprintf_free_buffer (temp);
+
+		  if (!empty_string_p (fundoc))
+		    bprintf (buffer, "VARIABLE: %s\n%s",
+			     words[windex], fundoc);
+		  else
+		    bprintf (buffer,
+			     "%s: No such command, function or variable.\n",
+			     words[windex]);
+		}
+
+	      xfree (fundoc);
+	    }
+	  free (words[windex]);
+	}
+      free (words);
+    }
+  else
+    {
+      for (i = 0; mdb_command_table[i].name; i++)
+	{
+	  bprintf (buffer, "%25s    %s\n", mdb_command_table[i].invocation,
+		   mdb_command_table[i].description);
+	}
+
+      bprintf (buffer, "  Or, type the name of a function or macro.\n");
     }
 
   result = buffer->buffer;
@@ -1490,7 +1591,14 @@ mdb_apropos (MDBArgs)
 		      char *symname = syms[i]->name;
 
 		      if (regexec (&re, symname, 1, offsets, 0) == 0)
-			bprintf (buffer, "%s::%s\n", pack->name, symname);
+			{
+			  if (pack == mhtml_function_package)
+			    bprintf (buffer, "Primitive: <%s>\n", symname);
+			  else if (pack == mhtml_user_keywords)
+			    bprintf (buffer, " Function: <%s>\n", symname);
+			  else
+			    bprintf (buffer, "%s::%s\n", pack->name, symname);
+			}
 		    }
 		}
 	    }
@@ -1528,13 +1636,14 @@ pf_include_handler (PFunArgs)
       char *canonicalized_name = (char *)NULL;
       char *incpref = pagefunc_get_variable ("%%::incpref");
       char *relpref = pagefunc_get_variable ("%%::relpref");
+      char *wr = (char *)NULL;
 
       if (!incpref) incpref = pagefunc_get_variable ("mhtml::include-prefix");
       if (!relpref) relpref = pagefunc_get_variable ("mhtml::relative-prefix");
 
       if (arg != (char *)NULL)
 	canonicalized_name =
-	  mhtml_canonicalize_file_name (arg, incpref, relpref);
+	  mhtml_canonicalize_file_name (arg, incpref, relpref, &wr);
 
       if (canonicalized_name != (char *)NULL)
 	{
@@ -1566,53 +1675,22 @@ pf_include_handler (PFunArgs)
 
 	  if (file_contents)
 	    {
-	      /* Manually insert the file instead of letting bprintf
-		 do it for us.  This is because the file could contain
-		 binary data, and then file->bindex wouldn't necessarily
-		 reflect the length of what was inserted. */
-	      if ((file_contents->bindex + page->bindex) >= page->bsize)
-		page->buffer = (char *)xrealloc
-		(page->buffer, (page->bsize += (file_contents->bindex + 100)));
+	      if (!verbatim_p)
+		{
+		  bprintf_insert (file_contents, 0, "<*parser*::push-file %s>",
+				  wr);
+		  bprintf (file_contents, "<*parser*::pop-file>");
+		}
 
-	      memmove (page->buffer + start + file_contents->bindex,
-		       page->buffer + start,
-		       (page->bindex + 1) - start);
-
-	      memcpy (page->buffer + start, file_contents->buffer,
-		      file_contents->bindex);
-	      page->bindex += file_contents->bindex;
+	      bprintf_insert_binary (page, start, file_contents->buffer,
+				     file_contents->bindex);
 
 	      if (verbatim_p)
 		*newstart += file_contents->bindex;
-#if defined (MHTML_INCLUDE_IS_RELATIVE)
-	      else if (!empty_string_p (relpref))
-		{
-		  char *temp = strstr (canonicalized_name, incpref);
-
-		  if (temp != (char *)NULL)
-		    {
-		      char *last;
-
-		      temp += strlen (incpref);
-		      last = strrchr (temp, '/');
-
-		      if (last != (char *)NULL)
-			{
-			  *last = '\0';
-
-			  bprintf_insert
-			    (page, start + file_contents->bindex,
-			     "<set-var %%::relpref=%s>", relpref);
-
-			  pagefunc_set_variable ("%%::relpref", temp);
-
-			}
-		    }
-		}
-#endif /* MHTML_INCLUDE_IS_RELATIVE */
 
 	      page_free_page (file_contents);
 	    }
+
 	  free (canonicalized_name);
 	}
     }
@@ -1677,6 +1755,66 @@ mdb_history (MDBArgs)
   return (result);
 }
 
+static char *
+mdb_cd (MDBArgs)
+{
+  register int i;
+  int changed = 0;
+  char *result = (char *)NULL;
+  char b[1024];
+
+  if (line && *line)
+    {
+      int len = strlen (line);
+      if ((line[len - 1] == '/') && (len > 1))
+	{
+	  while ((len > 1) && (line[len - 1] == '/'))
+	    line[--len] = '\0';
+	}
+
+      changed = (chdir (line) == 0);
+    }
+
+  if (changed)
+    {
+      /* Try to get smart here.  If the include-prefix contains "/docs/...",
+	 then the relative prefix is "/...", and the include-prefix goes up
+	 to "/docs". */
+      for (i = 0; line[i] != '\0'; i++)
+	if (strncasecmp (line + i, "/docs", 5) == 0)
+	  {
+	    if ((line[i + 5] == '\0') || (line[i + 5] == '/'))
+	      {
+		char *incpref = (char *)xmalloc (1 + (i + 5));
+		strncpy (incpref, line, i + 5);
+		incpref[i + 5] = '\0';
+		pagefunc_set_variable ("mhtml::include-prefix", incpref);
+		pagefunc_set_variable ("mhtml::relative-prefix", line + i +5);
+		sprintf (b, "Changed directory: [%s, %s]", incpref, line+i+5);
+		result = strdup (b);
+		free (incpref);
+		break;
+	      }
+	  }
+
+      if (!result)
+	{
+	  pagefunc_set_variable ("mhtml::include-prefix", line);
+	  pagefunc_set_variable ("mhtml::relative-prefix", "");
+	  sprintf (b, "Changed directory: [%s, %s]", line, "");
+	  result = strdup (b);
+	}
+    }
+
+  if (!result)
+    {
+      sprintf (b, "Cannot change to directory `%s'", line);
+      result = strdup (b);
+    }
+
+  return (result);
+}
+
 #if defined (__cplusplus)
 }
 #endif
