@@ -99,7 +99,11 @@ main (int argc, char *argv[])
   printf ("Compiled for (%s)\n", MHTML_SYSTEM_TYPE);
 
   initialize_mdb (argc, argv);
-  bootstrap_metahtml (1);
+
+  {
+    char *ignore = mhtml_evaluate_string ("<%%bootstrap-metahtml true>");
+    xfree (ignore);
+  }
 
   history_file = find_history_create (".mdb_history");
   history_truncate_file (history_file, 500);
