@@ -40,6 +40,11 @@
 #include "readline.h"
 #include "history.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 extern void *xmalloc (int), *xrealloc (void *, int);
 
 /* Not all systems declare ERRNO in errno.h... and some systems #define it! */
@@ -1363,3 +1368,7 @@ substring_member_of_array (char *string, char **array)
     }
   return (0);
 }
+
+#if defined (__cplusplus)
+}
+#endif
