@@ -37,6 +37,10 @@
 #include "rldefs.h"
 #include "readline.h"
 
+#if defined (__cplusplus)
+extern "C" {
+#endif
+
 typedef char **CompleterFunction (char *, int, int);
 typedef int qsort_func (const void *, const void *);
 typedef void Ignorer (char **);
@@ -1429,3 +1433,7 @@ memory_error_and_abort ()
   abort ();
 }
 #endif /* STATIC_MALLOC */
+
+#if defined (__cplusplus)
+}
+#endif
