@@ -47,6 +47,10 @@
 #include "readline.h"
 #include "history.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
 /* Not all systems declare ERRNO in errno.h... and some systems #define it! */
 #if !defined (errno)
 extern int errno;
@@ -93,11 +97,11 @@ static void readline_default_bindings (void);
 
 extern char *xmalloc (int), *xrealloc (char *, int);
 
-#if defined (hpux)
+#if defined (hpux) || defined (_AIX)
 char PC;
 char *BC;
 char *UP;
-#endif
+#endif /* hpux || _AIX */
 
 
 /* **************************************************************** */
@@ -2984,22 +2988,22 @@ rl_kill_text (int from, int to)
   /* If the last command was a kill, prepend or append. */
   if (last_command_was_kill && rl_editing_mode != vi_mode)
     {
-      char *old = rl_kill_ring[slot];
-      char *new = xmalloc (1 + strlen (old) + strlen (text));
+      char *old_kill = rl_kill_ring[slot];
+      char *new_kill = xmalloc (1 + strlen (old_kill) + strlen (text));
 
       if (from < to)
 	{
-	  strcpy (new, old);
-	  strcat (new, text);
+	  strcpy (new_kill, old_kill);
+	  strcat (new_kill, text);
 	}
       else
 	{
-	  strcpy (new, text);
-	  strcat (new, old);
+	  strcpy (new_kill, text);
+	  strcat (new_kill, old_kill);
 	}
-      free (old);
+      free (old_kill);
       free (text);
-      rl_kill_ring[slot] = new;
+      rl_kill_ring[slot] = new_kill;
     }
   else
     {
@@ -3459,9 +3463,14 @@ main ()
 
 #endif /* TEST */
 
+#if defined (__cplusplus)
+}
+#endif
+
 
 /*
  * Local variables:
  * compile-command: "gcc -g -traditional -I. -I.. -DTEST -o readline readline.c keymaps.o funmap.o history.o -ltermcap"
  * end:
  */
+
