@@ -68,6 +68,10 @@
 #define VI_COMMENT_BEGIN_DEFAULT "#"
 #endif
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
 extern char *xmalloc (int bytes), *xrealloc (char *p, int bytes);
 
 /* Variables imported from readline.c */
@@ -1314,3 +1318,7 @@ memory_error_and_abort ()
 #endif /* STATIC_MALLOC */
 
 #endif /* VI_MODE */
+
+#if defined (__cplusplus)
+}
+#endif
