@@ -98,6 +98,8 @@ main (int argc, char *argv[])
       exit_code = atoi (exit_code_text);
   }
 
+  page_process_page_internal (get_after_page_return_buffer ());
+
   return (exit_code);
 }
 
@@ -140,6 +142,37 @@ define_mhc_exit (void)
     }
 }
 
+static void
+pf_mhc_print (PFunArgs)
+{
+  char *item = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  if (!empty_string_p (item))
+    fprintf (stdout, "%s", item);
+
+  xfree (item);
+}
+
+static PFunDesc printdef = { "MHC::PRINT", 0, 0, pf_mhc_print };
+
+static void
+define_mhc_print (void)
+{
+  static int defined = 0;
+
+  if (!defined)
+    {
+      Symbol *sym;
+
+      if (mhtml_function_package == (Package *)NULL)
+	mhtml_function_package = symbol_get_package ("*META-HTML*");
+
+      sym = symbol_intern_in_package (mhtml_function_package, "MHC::PRINT");
+      sym->type = symtype_FUNCTION;
+      sym->values = (char **)&printdef;
+      defined = 1;
+    }
+}
+
 static void
 prep_for_page_process (PAGE *page, int argc, char *argv[])
 {
@@ -147,8 +180,10 @@ prep_for_page_process (PAGE *page, int argc, char *argv[])
 
   if (call_bootstrap)
     {
-      bootstrap_metahtml (0);
+      char *ignore = mhtml_evaluate_string ("<%%bootstrap-metahtml>");
       define_mhc_exit ();
+      define_mhc_print ();
+      xfree (ignore);
     }
 
   if (page && page->buffer)
@@ -173,6 +208,14 @@ prep_for_page_process (PAGE *page, int argc, char *argv[])
 
       /* Now set Meta-HTML variables, and variables from the environment. */
       forms_input_data (argc, argv);
+
+      /* The call to forms_input_data () bashed default::program-arguments.
+	 Replace that with the arguments to our script. */
+      {
+	  char *ignore = mhtml_evaluate_string
+	    ("<copy-var mhtml::program-arguments default::program-arguments>");
+	  xfree (ignore);
+	}
     }
 }
 
@@ -181,9 +224,19 @@ parse_program_args (int argc, char *argv[])
 {
   int arg_index = 1;
   char *filename = (char *)NULL;
+  Symbol *program_args = symbol_intern ("mhtml::program-arguments");
 
   progname = argv[0];
-  pagefunc_set_variable ("mhc::script-name", progname);
+  pagefunc_set_variable ("mhc::mhc-executatble", progname);
+
+  /* Remember the argv array in a symbol reserved for that purpose. */
+  {
+    register int i;
+    Symbol *s = symbol_intern ("mhc::argv");
+
+    for (i = 0; i < argc; i++)
+      symbol_add_value (s, argv[i]);
+  }
 
   while (arg_index < argc)
     {
@@ -217,15 +270,13 @@ parse_program_args (int argc, char *argv[])
 	    }
 	}
       else if (!filename && (*arg != '-'))
-	filename = arg;
-      else if (filename)
 	{
-	  static Symbol *sym = (Symbol *)NULL;
-
-	  if (!sym) sym = symbol_intern ("mhtml::program-arguments");
-
-	  symbol_add_value (sym, arg);
+	  filename = arg;
+	  symbol_add_value (program_args, filename);
+	  pagefunc_set_variable ("mhc::script-name", filename);
 	}
+      else if (filename)
+	symbol_add_value (program_args, arg);
       else
 	usage ();
     }
