@@ -3,7 +3,7 @@
 
 /*  Copyright (c) 1997 Brian J. Fox
     Author: Brian J. Fox (bfox@ai.mit.edu) Tue Sep 30 09:02:13 1997.  */
-#include "language.h"
+#include "modules.h"
 
 #if defined (__cplusplus)
 extern "C"
@@ -17,34 +17,46 @@ static PFunDesc ftab[] =
   { (char *)NULL,	0, 0, (PFunHandler *)NULL }
 };
 
-void
-module_initialize (void)
-{
-  static int called = 0;
+MODULE_INITIALIZE ("serverfuncs", ftab)
 
-  if (!called)
-    {
-      register int i;
-      Symbol *sym, *funcnames;
+DEFINE_SECTION (META-HTML-TCP/IP-SERVERS, tcp/ip;port;telnet,
+"<Meta-HTML> provides an extremely convenient path for creating a TCP/IP
+server which will listen on a particular port, and handle line-based
+commands.
 
-      called++;
-      funcnames = symbol_intern ("modules::syms-of-serverfuncs");
+Typically, a <b>mhc</b> script is used to start the server process -- it
+loads the <code>serverfuncs</code> module, creates a server process, binds
+<code>*standard-input*</code> and <code>*standard-output</code> to the
+specified port, and waits for connections.
 
-      /* Install the names and pointers. */
-      for (i = 0; ftab[i].tag != (char *)NULL; i++)
-	{
-	  sym = symbol_intern_in_package (mhtml_function_package, ftab[i].tag);
-	  symbol_add_value (funcnames, ftab[i].tag);
-	  sym->type = symtype_FUNCTION;
-	  sym->values = (char **)(&ftab[i]);
-	}
-    }
-}
+When a connection is received, the server process forks and executes the
+function that you have specified to run.  Upon exit from that function,
+the connection is closed.
 
-void _init (void) { module_initialize (); }
+The libraries that are provided in the <code>modules/serverfunc-examples</code>
+tagsets directory do most of the work for implementing a complete server --
+you need only to write the commands which implement the specific functionality
+that you require.",
+"The type of server that is implemented by the <code>tagsets/server.mhtml</code>
+code expect to interact in an ASCII based conversation mode; each request
+from the client is a single line of ASCII text, and your server is expected
+to produce a response (which can be multiple lines, or a single line, or
+anything that your protocol implements).
 
-/* PACKAGE_INITIALIZER (initialize_server_functions) */
-DOC_SECTION (PROCESS-OPERATORS)
+The code in <code>tagsets/server.mhtml</code> implement a few commands for you
+already, including <code>quit</code>, <code>help</code>, and <code>login</code>.
+
+You implement new commands by simply writing a function whose name is
+<code>COMMAND::</code><i>function-name</i></code>, where <i>function-name</i>
+is the exact string that the client should send to invoke the command.
+
+The convenience functions <tag server::put-line> and <tag server::get-line>
+write and read newline terminated lines of text to and from the client
+respectively.
+
+If you are interested in writing a TCP/IP based server, we would suggest that
+you read the source code to the <code>tagsets/server.mhtml</code> library,
+and peruse the examples.")
 
 DEFUNX (pf_server::make_server, &key hostname &rest start-fun port,
 "Create a server process which will listen on <var port> for incoming
