@@ -1175,6 +1175,14 @@ loop_over_connections (void)
 			    setgid (entry->pw_gid);
 
 			  setuid (entry->pw_uid);
+
+			  /* On some machines, calling setuid twice is just
+			     like calling seteuid.  Well, we'll see. */
+#if defined (HAVE_SETEUID)
+			  seteuid (entry->pw_uid);
+#else
+			  setuid (entry->pw_uid);
+#endif
 			}
 		    }
 #endif /* HAVE_GETPWNAM */
@@ -1537,8 +1545,11 @@ loop_over_connections (void)
 	  if (!debug_mode)
 	    {
 	      shutdown (connection, 2);
+	      page_process_page_internal (get_after_page_return_buffer ());
 	      _exit (0);
 	    }
+	  else
+	    page_process_page_internal (get_after_page_return_buffer ());
 	}
     }
 }
